#! /usr/bin/env python
import pygtk; pygtk.require("2.0")
import gobject, gtk
import kernel, CORBA, bonobo
import gnome
import textview_display as disp
#import gtkhtml2_display as disp
import entry
import varbrowser
import numexp_client
import gc
import os.path
import Numexp
import locale



class Console(object):

    control = None

    def __init__(self):

        self.simplification = False
	self._vpaned = gtk.VPaned()
	self._vpaned.show()
	self._hpaned = gtk.HPaned()
	self._hpaned.show()

	self.display = disp.Display()

	self._vpaned.pack1(self.display.get_widget(), True, False)

	self.entry = entry.Entry()
	self.entry.widget.show()

	frame = gtk.Frame()
	frame.set_shadow_type(gtk.SHADOW_IN)
	frame.show()
	frame.add(self.entry.widget)
	self._vpaned.pack2(frame, False, False)

	self._hpaned.pack1(self._vpaned, True, False)
	self._varbrowser = varbrowser.View(varbrowser.Model(), self)
	self._hpaned.pack2(self._varbrowser.widget, True, False)
	try:
	    self.entry.history.load(file(os.path.expanduser("~/.gnumexp_history")))
	except IOError:
	    pass
	self.entry.connect("text-entered", self.__entry_text_entered)
	self.entry.connect("completions-request", self.__entry_completions_request)
	self.kernel = kernel.get_kernel()
	self._varbrowser.model.update_namespace("::", self.kernel)

	self.client = numexp_client.NumexpClient(self, self.kernel)
	self.kernel.setClient(self.client._this())
	self.kernel.registerNamespaceListener("::")

	self.entry.grab_focus()
	self.widget.connect("destroy", self.__destroy)

	self.entry.widget.connect("key-press-event", self.__entry_key_press)

    def __entry_key_press(self, widget, event):
	if event.state == 0 and event.keyval == gtk.keysyms.Page_Up:
	    self.display.page_up()
	elif event.state == 0 and event.keyval == gtk.keysyms.Page_Down:
	    self.display.page_down()
	else:
	    return False
	return True
    
    def __destroy(self, *args):
	self.client.unref()
	self.kernel.unref()
	self.entry.history.save(file(os.path.expanduser("~/.gnumexp_history"), "w"))

    def set_transient_for(self, win):
	if self.control is not None:
	    self.control.set_transient_for(win)
	else:
	    win.set_transient_for(self.widget.get_toplevel())
	    
    def _eval_user_input(self, text):
	tree = kernel.Element(self.kernel, self.kernel.parseString(text))
        if self.simplification:
            result = kernel.Element(self.kernel, self.kernel.elementSimplify(tree.id))
        else:
            result = kernel.Element(self.kernel, self.kernel.elementEval(tree.id))
	self.kernel.defineVariable("::ans", result.id)
	return self.kernel.elementStr(result.id)

    def _eval_user_input_mathml(self, text):
	tree = kernel.Element(self.kernel, self.kernel.parseString(text))
	input_mathml = self.kernel.elementToMathML(tree.id)
        if self.simplification:
            result = kernel.Element(self.kernel, self.kernel.elementSimplify(tree.id))
        else:
            result = kernel.Element(self.kernel, self.kernel.elementEval(tree.id))
	self.kernel.defineVariable("::ans", result.id)
	output_mathml = self.kernel.elementToMathML(result.id)
	return (input_mathml, output_mathml)

    def __entry_text_entered(self, entry, text):
	if text:
	    self.read_eval_print(text)

    def read_eval_print(self, text):
	try:
	    if self.__mathml_output:
		input_mathml, output_mathml =\
			      self._eval_user_input_mathml(text)
	    else:
		result = self._eval_user_input(text)
	except CORBA.UserException, ex:
	    self.samples = None
	    err = str(ex.__class__) + ": " + ex.detail
	    dialog = gtk.MessageDialog(
		flags          = gtk.DIALOG_DESTROY_WITH_PARENT,
		type           = gtk.MESSAGE_ERROR,
		buttons        = gtk.BUTTONS_OK,
		message_format = err)
	    dialog.set_property("has-separator", gtk.FALSE)
	    self.set_transient_for(dialog)
	    dialog.run()
	    dialog.destroy()
	    return
	if self.__mathml_output:
	    self.display.display_qa_mathml(
		"<math display='block'>%s</math>" % input_mathml,
		"<math display='block'>%s</math>" % output_mathml)
	else:
	    self.display.display_qa(text, result)

    def get_widget(self):
	return self._hpaned

    widget = property(get_widget, None, None,
		      "return the entry toplevel container widget")

    def display_help_on(self, symbol):
	print "display_help_on(\"%s\")" % symbol
	info = self.kernel.getDefinitionInfo(symbol)
	self.display.display_help(symbol, info.description)

    def __entry_completions_request(self, entry, prefix):
	# convert relative paths to absolute ones, considering the cwd
	if len(prefix) < 2 or prefix[:2] != '::':
	    prefix = self.client.cwd + prefix
	    need_strip_root = True
	else:
	    need_strip_root = False
	
	namespace = prefix[:prefix.rfind("::") + 2]
	#print "completing using namespace ", namespace
	try:
	    completions = self._varbrowser.model.get_namespace_listing(namespace)
	except ValueError:
	    return None
	#print "completions:", completions
	
	if need_strip_root:
	    completions = [comp[2:] for comp in completions if comp.startswith(prefix)]
	else:
	    completions = [comp for comp in completions if comp.startswith(prefix)]
	completions.sort(locale.strcoll)
	return completions

    def set_varbrowser_active(self, active):
	self._varbrowser.widget.set_property("visible", active)

    def set_mathml_output(self, active):
	self.__mathml_output = active

    def clear(self):
        self.display.clear()

    def set_simplification(self, state):
        self.simplification = state

def test():
    gnome.program_init("gnumexp", "test")
    window = gtk.Window()
    window.connect("destroy", lambda w: bonobo.main_quit())
    window.set_default_size(gtk.gdk.screen_width()*2/3,
			    gtk.gdk.screen_height()*2/3)
    console = Console()
    window.add(console.widget)
    window.show_all()
    bonobo.main()


if __name__ == '__main__':
    #gc.set_debug(gc.DEBUG_LEAK)
    test()
