from nxplot import *
from misc import *
from math import fabs
import gtk, gtk.gdk
from gettext import gettext as _

_AXIS_LABELS_MULT = 10e-3

class AxisLineOverlay(NxpCanvasOverlay):
    def __init__(self):
	NxpCanvasOverlay.__init__(self)
	self.connect("draw", self.__class__.real_draw)

    def real_draw(self, drawable, gc, t):
	drawable.draw_rectangle(gc, 1, self.bbox.x, self.bbox.y,
				self.bbox.width,
				self.bbox.height)



class Ticks:
    """Class to hold tick marks spacing and position values,
    already mapped to the drawing rectangle"""

    def __init__(self, start, div, subdiv, limit):
	self.start  = start		# value of first tick mark
	self.div    = div		# size of first-level division
	self.subdiv = subdiv		# size of second-level division
	self.limit  = limit		# value of last tick mark


class Axis(CanvasContainerBase):
    ticks        = None			# ticks in function units
    ticks_plunit = None			# ticks in plot units
    menu         = None

    def popup_menu(self, event):
	if self.menu is None:
	    menu = gtk.Menu()

	    item = gtk.ImageMenuItem(stock_id=gtk.STOCK_PROPERTIES)
	    item.show()
	    item.connect("activate", self.display_properties_dialog)
	    menu.append(item)
	    
	    self.menu = menu
	    
	self.menu.popup(parent_menu_shell=None, parent_menu_item=None,
			func=None, button=event.button,
			activate_time=event.time)
	
    def display_properties_dialog(self, *args):
	pass

    # << XML load/save >>
    def to_xml(self, doc):
	elem = doc.createElement(self.axis_xml_name)
	elem.setAttribute("position", str(self.pos_fn))
	return elem

    def from_xml(self, elem):
	assert isinstance(elem, dom.Element)
	assert elem.tagName == self.axis_xml_name
	pos = float(elem.getAttribute("position"))
	self.change_position(pos)


class AxisLine(CanvasObjectBase):
    def __init__(self, attr, width_scale, pos):
	"""
	@attr: LineAttr
	@width_scale: float
	@pos: (float) position (either X or Y) in plot coordinates
	"""
	super(AxisLine, self).__init__()
	self.set_values(attr, width_scale, pos)
	self.set_property('interaction-mask', 0)
	self.set_property('whole-area-interaction', 0)
	self.overlay = AxisLineOverlay()
	self.overlay.set_bbox(0, 0, 0, 0)
	self.connect("notify::canvas", self.__notify_canvas)
	self.cursor = None

    def real_size_request(self):
	return NxplotPoint(self.linewidth, self.linewidth)

    def real_enter(self):
	self.canvas.set_cursor(self.cursor)

    def real_leave(self):
	self.canvas.set_cursor(None)

    def __notify_canvas(self, *args):
	self.overlay.set_canvas(self.canvas)

    def set_values(self, attr, width_scale, pos):
	self.attr        = attr
	self.width_scale = width_scale
	self.pos         = None # position in canvas coordinates
	self.linewidth   = self.attr.get_linewidth(self.width_scale)
	self.queue_draw()

    def set_pos(self, pos):
	self.pos = pos
	self.queue_draw()

    def real_button_press(self, event):
	if self.drag_state: return 1
	if event.button == 3:
	    self.container.popup_menu(event)
	    return
	if event.button != 1: return 0
	self.drag_state = 1
	self.overlay_geometry_save = self.overlay_geometry
	self.drag_origin = event.x, event.y
	self.overlay.hide()
	self.overlay.set_bbox(*map(int, self.overlay_geometry))
	self.overlay.show()
	self.container.pointer_grab(self, 1)
	return 1


class HorizAxisLine(AxisLine):

    def __init__(self, *args):
	AxisLine.__init__(self, *args)
	self.drag_state = 0
	self.cursor = gtk.gdk.Cursor(gtk.gdk.SB_V_DOUBLE_ARROW)

    def real_configure(self, bbox, interaction):
	if interaction != Interaction.MOVE_RESIZE:
	    return
	pl = self.get_plotter()
	if isinstance(pl, NxplotGP):
	    return
	self.y  = (bbox.y1 + bbox.y2) / 2
	self.x1 = bbox.x1
	self.x2 = bbox.x2
	x1, y1 = pl.space2window(bbox.x1, bbox.y1)
	x2, y2 = pl.space2window(bbox.x2, bbox.y2)
	self.overlay_geometry = (x1, y2, 0.5 + x2 - x1, 0.5 + y1 - y2)
	#print self.overlay_geometry
	#self.overlay_geometry = (self.wbbox.x, self.wbbox.y,
	#			 self.wbbox.width, self.wbbox.height)

    def real_draw(self, pl):
	self.attr.apply_to(pl, self.width_scale)
	pl.endpath()
	pl.line(self.x1, self.y, self.x2, self.y)
	pl.endpath()


    def real_motion_notify(self, event):
	if not self.drag_state: return 0
	self.overlay.hide()
	delta = (event.y - self.drag_origin[1])
	self.overlay_geometry = (self.overlay_geometry_save[0],
				 self.overlay_geometry_save[1] + delta,
				 self.overlay_geometry_save[2],
				 self.overlay_geometry_save[3])
	self.overlay.set_bbox(*map(int, self.overlay_geometry))
	self.overlay.show()


    def real_button_release(self, event):
	if not self.drag_state: return 1
	self.drag_state = 0
	self.overlay.hide()
	self.overlay_geometry = self.overlay_geometry_save
	self.container.pointer_ungrab()
	pl = self.get_plotter()
	x0, y0 = pl.window2space(*self.drag_origin)
	x1, y1 = pl.window2space(event.x, event.y)
	delta = y1 - y0
	self.set_pos(self.pos + delta)
	self.container.line_position_changed(self.pos)


class HorizAxis(Axis):
    axis_xml_name = "haxis"
    
    def __init__(self, pos, attr, width_scale):
	CanvasContainerBase.__init__(self)
	self.width_scale = width_scale
	self.pos = pos
	self.pos_fn = pos  # position in 'function' coordinates
	self.attr = attr
	self.line = HorizAxisLine(attr, width_scale, pos)
	self.line.set_container(self)
	self.vaxis_pos = None
	self.update_derived_parameters()

    def line_position_changed(self, pos):
	self.pos = pos
	self.container.haxis_position_changed(pos)
	self.container.axis_geometry_update_request(self)

    def set_pos(self, pos):
	self.pos = pos
	self.line.set_pos(pos)

    def update_derived_parameters(self):
	# dy is the half the length of the tick marks
	self.dy       = self.attr.get_linewidth(self.width_scale)*3
	self.fontsize = self.width_scale*30
	self.fontname = "Sans Regular"

    def real_size_request(self):
	# the width is irrelevant
	return NxplotRect(0, self.dy*2 + self.fontsize)

    def geometry_request(self):	# an ad-hoc geometry negotiation interface
	y2 = self.pos + self.dy
	y1 = self.pos - (self.dy + self.fontsize)
	return None, None, y1, y2

    def real_configure(self, bbox, interaction):
	if interaction != Interaction.MOVE_RESIZE:
	    return
	self.y  = bbox.y2 - self.dy	# position of the line
	self.x1 = bbox.x1
	self.x2 = bbox.x2

	lwbbox = self.line.size_request()
	if lwbbox is None: return
	
	half_lw = lwbbox.y/2
	self.line.configure(NxplotRect(self.x1, self.y - half_lw,
				       self.x2, self.y + half_lw),
			    interaction)

    def real_get_children(self):
	return NxplotList((self.line,))

    def real_draw(self, pl):
	start  = self.ticks_plunit.start
	div    = self.ticks_plunit.div
	if not div > 0: return
	subdiv = self.ticks_plunit.subdiv
	limit  = self.ticks_plunit.limit
	y0 = self.y
	dy = self.dy
	y1, y2 = y0 - dy, y0 + dy
	self.attr.apply_to(pl, self.width_scale)
	pl.textangle(0.0)
	pl.fontsize(self.fontsize)
	pl.fontname(self.fontname)
	
	x = start - div
	xr = self.ticks.start - self.ticks.div
	while x < limit:
	    if x >= start and fabs(x - self.vaxis.pos) >= div/2:
		self.attr.apply_to(pl, self.width_scale)
		pl.line(x, y1, x, y2)
		pl.move(x, y1)
		pl.pencolor(0, 0, 0)
		if math.fabs(xr/self.ticks.div) < 1e-6:
		    pl.label(HAlign.CENTER, VAlign.TOP, "0")
		else:
		    pl.label(HAlign.CENTER, VAlign.TOP, str(xr))
	    x  += div
	    xr += self.ticks.div

    def change_position(self, newpos_fn):
	self.container.set_haxis_position(newpos_fn)

    def display_properties_dialog(self, *args):
	plotarea = self.get_toplevel()
	plotarea.view.create_haxis_properties_dialog()


class VertAxisLine(AxisLine):

    def __init__(self, *args):
	AxisLine.__init__(self, *args)
	self.drag_state = 0
	self.cursor = gtk.gdk.Cursor(gtk.gdk.SB_H_DOUBLE_ARROW)


    def real_configure(self, bbox, interaction):
	if interaction != Interaction.MOVE_RESIZE:
	    return
	pl = self.get_plotter()
	if isinstance(pl, NxplotGP):
	    return
	self.x  = (bbox.x1 + bbox.x2) / 2
	self.y1 = bbox.y1
	self.y2 = bbox.y2
	x1, y1 = pl.space2window(bbox.x1, bbox.y1)
	x2, y2 = pl.space2window(bbox.x2, bbox.y2)
	self.overlay_geometry = (x1, y2, 0.5 + x2 - x1, 0.5 + y1 - y2)
	#print self.overlay_geometry
	#self.overlay_geometry = (self.wbbox.x, self.wbbox.y,
	#			 self.wbbox.width, self.wbbox.height)

    def real_draw(self, pl):
	self.attr.apply_to(pl, self.width_scale)
	pl.endpath()
	pl.line(self.x, self.y1, self.x, self.y2)
	pl.endpath()

    def real_motion_notify(self, event):
	if not self.drag_state: return 0
	self.overlay.hide()
	delta = (event.x - self.drag_origin[0])
	self.overlay_geometry = (self.overlay_geometry_save[0] + delta,
				 self.overlay_geometry_save[1],
				 self.overlay_geometry_save[2],
				 self.overlay_geometry_save[3])
	self.overlay.set_bbox(*map(int, self.overlay_geometry))
	self.overlay.show()

    def real_button_release(self, event):
	if not self.drag_state: return 1
	self.drag_state = 0
	self.overlay.hide()
	self.overlay_geometry = self.overlay_geometry_save
	self.container.pointer_ungrab()
	pl = self.get_plotter()
	x0, y0 = pl.window2space(*self.drag_origin)
	x1, y1 = pl.window2space(event.x, event.y)
	delta = x1 - x0
	self.set_pos(self.pos + delta)
	self.container.line_position_changed(self.pos)


class VertAxis(Axis):
    axis_xml_name = "vaxis"

    def __init__(self, pos, attr, width_scale):
	CanvasContainerBase.__init__(self)
	self.width_scale = width_scale
	self.pos = pos
	self.pos_fn = pos  # position in 'function' coordinates
	self.attr = attr
	self.line = VertAxisLine(attr, width_scale, pos)
	self.line.set_container(self)
	self.haxis_pos = None
	self.update_derived_parameters()

    def line_position_changed(self, pos):
	self.pos = pos
	self.container.vaxis_position_changed(pos)
	self.container.axis_geometry_update_request(self)

    def set_pos(self, pos):
	self.pos = pos
	self.line.set_pos(pos)

    def update_derived_parameters(self):
	# dx is the half the length of the tick marks
	self.dx       = self.attr.get_linewidth(self.width_scale)*3
	self.fontsize = self.width_scale*30
	self.fontname = "Sans Regular"

    def real_size_request(self):
	# the height is irrelevant
	return NxplotRect(self.dx*2 + self.fontsize, 0)

    def geometry_request(self):	# an ad-hoc geometry negotiation interface
	x2 = self.pos + self.dx
	x1 = self.pos - (self.dx + self.fontsize)
	return x1, x2, None, None

    def real_configure(self, bbox, interaction):
	if interaction != Interaction.MOVE_RESIZE:
	    return
	self.x  = bbox.x2 - self.dx	# position of the line
	self.y1 = bbox.y1
	self.y2 = bbox.y2

	lwbbox = self.line.size_request()
	if lwbbox is None: return
	
	half_lw = lwbbox.x/2
	self.line.configure(NxplotRect(self.x - half_lw, self.y1,
				       self.x + half_lw, self.y2),
			    interaction)

    def real_get_children(self):
	return NxplotList((self.line,))

    def real_draw(self, pl):
	start  = self.ticks_plunit.start
	div    = self.ticks_plunit.div
	if not div > 0: return
	subdiv = self.ticks_plunit.subdiv
	limit  = self.ticks_plunit.limit
	x0 = self.x
	dx = self.dx
	x1, x2 = x0 - dx, x0 + dx
	pl.textangle(90.0)
	pl.fontsize(self.fontsize)
	pl.fontname(self.fontname)

	y = start - div
	yr = self.ticks.start - self.ticks.div
	while y < limit:
	    if y >= start and fabs(y - self.haxis.pos) >= div/2:
		self.attr.apply_to(pl, self.width_scale)
		pl.line(x1, y, x2, y)
		pl.move(x1, y)
		pl.pencolor(0, 0, 0)
		if math.fabs(yr/self.ticks.div) < 1e-6:
		    pl.label(HAlign.CENTER, VAlign.BOTTOM, "0")
		else:
		    pl.label(HAlign.CENTER, VAlign.BOTTOM, str(yr))
	    y  += div
	    yr += self.ticks.div

    def change_position(self, newpos_fn):
	self.container.set_vaxis_position(newpos_fn)

    def display_properties_dialog(self, *args):
	plotarea = self.get_toplevel()
	plotarea.view.create_vaxis_properties_dialog()

