#! /usr/bin/env python
import sys, os
import bonobo, bonobo.ui, gconf, gobject, CORBA
import gnumexp_conf
import functionobject, scatterobject, parametricobject
import printing
import xml.dom.minidom as dom
from view     import *
from StringIO import StringIO
from xygraphC import *
import persist
import gtk.gdk
import warnings
import gzip
from GNumexp_Common import util

def _foo():
    reset_lc_numeric() # see comment in XYGraphView::load_from
    return 0
import gobject
gobject.idle_add(_foo)

import user

icon = gtk.gdk.pixbuf_new_from_file(os.path.join(gnumexp_conf.ICONDIR, "gnumexp_icon.png"))
gtk.window_set_default_icon_list(icon); del icon


GRID_TOGGLE_KEY   = "/apps/gnumexp/XYGraph/grid_enabled"
AXIS_TOGGLE_KEY   = "/apps/gnumexp/XYGraph/axis_enabled"
LEGEND_TOGGLE_KEY = "/apps/gnumexp/XYGraph/legend_enabled"

# Bonobo::PropertyFlags
#_PROPERTY_READABLE        = 1;
#_PROPERTY_WRITEABLE       = 2;
#_PROPERTY_NO_LISTENING    = 4;
#_PROPERTY_NO_AUTONOTIFY   = 8;
#_PROPERTY_NO_PERSIST      = 16;

def verb_ignore(*args): pass


class XYGraphControl(bonobo.ui.Control):
    def __init__(self, dnotify=None):
	self.xygraph = xygraph = XYGraphView()
	self.xygraph.control = self
	bonobo.ui.Control.__init__(self, xygraph.get_toplevel_widget())
	self.connect('set-frame', self.__set_frame_cb)
	self.connect('activate',  self.real_activate)
	self.__dnotify = dnotify
	self.__destroyed = 0

	# Other interfaces
	self._add_persist_stream()
	self._add_persist_uri()
	self._add_item_handler()
	self._add_property_bag()


    def load_from_string(self, content):
	f = gzip.GzipFile(fileobj=StringIO(content))
	try:
	    f.read(1)
	    compressed = True
	except IOError: # Not a gzipped file
	    compressed = False
	f = StringIO(content)
	if compressed:
	    f = gzip.GzipFile(fileobj=f)
	self.xygraph.load_from(f)
	self.__persist_stream.dirty = False

    def save_to_string(self):
	content = StringIO()
	f = gzip.GzipFile(mode="w", fileobj=content)
	self.xygraph.save_to(f)
	f.close()
        return content.getvalue()

    ## ------- Bonobo::PersistStream interface  -------
    def __persist_stream_load(self, stream, content_type):
	print "** __persist_stream_load **"
	if content_type != "application/x-xygraph":
	    raise ValueError("Cannot handle content_type " + content_type)
	content = stream.read(bonobo.stream_client_get_length(stream))
	self.load_from_string(content)

    def __persist_stream_save(self, stream, content_type):
	print "** __persist_stream_save **"
	if content_type != "application/x-xygraph":
	    raise ValueError("Cannot handle content_type " + content_type)
        data = self.save_to_string()
	self.__persist_stream.dirty = False
	stream.write(data)

    def _add_persist_stream(self):
	_persist = persist.PersistStream(save_cb=self.__persist_stream_save,
					 load_cb=self.__persist_stream_load)
	_persist.add_content_type("application/x-xygraph")
	print "PersistStream:", _persist.get_bonobo_object()
	self.add_interface(_persist.get_bonobo_object())
	self.__persist_stream = _persist

    ## ------- Bonobo::PersistFile interface  -------
    
    def __persist_uri_load(self, data):
	print "** __persist_uri_load **"
	self.load_from_string(data)

    def __persist_uri_save(self):
	print "** __persist_uri_save **"
        return self.save_to_string()

    def _add_persist_uri(self):
	_persist = persist.PersistURI(save_cb=self.__persist_uri_save,
                                      load_cb=self.__persist_uri_load,
                                      parent=self)
	_persist.content_types.append("application/x-xygraph")
	self.add_interface(_persist.get_bonobo_object())
	self.__persist_stream = _persist

    ## ------- Bonobo::ItemHandler interface WORKS with latest (CVS) gnome-python  -------
    def __item_handler_enum_objects(self, *args):
	print "__item_handler_enum_objects: ", args

    def __item_handler_get_object(self, item_handler, item_name,
				  only_if_exists, ev):
	#print "__item_handler_get_object", item_name
	if item_name[0] == '<':
	    # looks like it's an xml fragment; lets parse it
	    item_name = bonobo.moniker_util_unescape(item_name, len(item_name))
	    doc = dom.parseString(item_name)
	    if doc.documentElement.tagName == 'plotarea':
		xygraphC.reset_lc_numeric() # see comment in XYGraphView::load_from
		self.xygraph.from_xml_plotarea(doc.documentElement)
		gobject.idle_add(lambda xy: xy.auto_vertical_fit(), self.xygraph)
	obj = self.corba_objref()
	obj.ref()
	return obj
	
    def _add_item_handler(self):
	item_handler = bonobo.ItemHandler(self.__item_handler_enum_objects,
					  self.__item_handler_get_object)
	self.add_interface(item_handler)
	self.__item_handler = item_handler

    ## ------- Bonobo::PropertyBag interface -------
    __PB_PROPERTY_ADD_PLOT_OBJECT = 1
    def __property_bag_set_fn(self, pbag, arg, arg_id, *args):
	#print "property_bag_set_fn:", arg, arg_id
	#print "extra args:", args
	if arg_id == self.__PB_PROPERTY_ADD_PLOT_OBJECT:
	    print "add_plot_object property set to '%s'" % arg.value()
	    doc = dom.parseString(arg.value())
	    if doc.documentElement.tagName == 'plotarea':
		xygraphC.reset_lc_numeric() # see comment in XYGraphView::load_from
		self.xygraph.from_xml_plotarea(doc.documentElement, 0)
		gobject.idle_add(lambda xy: xy.auto_vertical_fit(), self.xygraph)    

    def __property_bag_get_fn(self, *args):
	print "args:", args
	if arg_id == self.__PB_PROPERTY_ADD_PLOT_OBJECT:
	    return ""
	
    def _add_property_bag(self):
	pbag = bonobo.PropertyBag(self.__property_bag_get_fn,
				  self.__property_bag_set_fn)
	pbag.add("add_plot_object", self.__PB_PROPERTY_ADD_PLOT_OBJECT,
		 CORBA.TC_string, CORBA.Any(CORBA.TC_string, ""),
		 "The XML fragment (root is <plotarea>) of an"
		 " object to be added to the plot",
		 bonobo.PROPERTY_WRITEABLE|bonobo.PROPERTY_READABLE)
	self.add_interface(pbag)
	self.__pbag = pbag
	#self.set_properties(pbag.corba_objref())

    ## ------- continuation of Bonobo::Control -------
    def real_activate(self, *args):
	print "XYGraph control activated"
	
    def __set_frame_cb(self, control):
	frame = control.get_control_frame()
	if frame is None:
	    if self.__destroyed: return
	    self.__destroyed = 1
	    uic = control.get_ui_component()
	    uic.unset_container()
	    if self.__dnotify is not None:
		self.__dnotify(self)
	else:
	    ui_component = self.get_ui_component()
            ui_container = self.get_remote_ui_container()
	    ui_component.set_container(ui_container)
            ui_container.unref()
	    print "ui_component.set_container(self.get_remote_ui_container())"
	    bonobo.ui.util_set_ui(ui_component, '',
				  gnumexp_conf.UIDIR + '/Numexp_XYGraph-ui.xml',
				  'numexp-xygraph')
	    verbs = [ ('AddObject', 	self.add_object_callback),
		      ('ObjectList', 	self.object_list_callback),
		      ('ZoomIn', 	self.zoom_in_callback),
		      ('ZoomOut', 	self.zoom_out_callback),
		      ('Refresh', 	self.refresh_cb),
		      ('FilePrint', 	self.print_cb),
		      ('EditClear', 	self.clear_cb),
		      ('EditProperties',self.edit_properties_cb),
		      ('EditHAxis', 	self.edit_haxis_cb),
		      ('EditVAxis', 	self.edit_vaxis_cb),
		      ('EditCut', 	self.edit_cut_cb),
		      ('EditCopy', 	self.edit_copy_cb),
		      ('EditPaste', 	self.edit_paste_cb),
                      ('ZoomArea', 	verb_ignore),
                      ('GridToggle', 	verb_ignore),
                      ('AxisToggle', 	verb_ignore),
                      ('LegendToggle', 	verb_ignore),
		      ]
	    ui_component.add_verb_list(verbs)
	    self._read_settings(ui_component)

            ui_component.add_listener('ZoomArea', 	self.zoom_area_callback)
            ui_component.add_listener('GridToggle', 	self.grid_toggle_cb)
            ui_component.add_listener('AxisToggle', 	self.axis_toggle_cb)
            ui_component.add_listener('LegendToggle', 	self.legend_toggle_cb)
            
	    #if not self.xygraph.get_plot_objects():
	    #    self.set_transient_for(self.xygraph.show_add_object_dialog().dialog)
	    if not printing.available:
		ui_component.set_prop('/commands/FilePrint', 'sensitive', '0')

    def _read_settings(self, uic):
	"""Set the initial toggle states from GConf settings"""
	gc = gconf.client_get_default()

	# Grid toggle
	val = gc.get(GRID_TOGGLE_KEY)
	if val is not None: val = util.bool_str(val.get_bool())
	else: val = "1"
	uic.set_prop("/commands/GridToggle", "state", val)
	self.grid_toggle_cb(uic, "GridToggle", 0, val)

	# Axis toggle
	val = gc.get(AXIS_TOGGLE_KEY)
	if val is not None: val = util.bool_str(val.get_bool())
	else: val = "1"
	uic.set_prop("/commands/AxisToggle", "state", val)
	self.axis_toggle_cb(uic, "AxisToggle", 0, val)

	# Legend toggle
	val = gc.get(LEGEND_TOGGLE_KEY)
	if val is not None: val = util.bool_str(val.get_bool())
	else: val = "1"
	uic.set_prop("/commands/LegendToggle", "state", val)
	self.legend_toggle_cb(uic, "LegendToggle", 0, val)

	# It is assumed the selection is initially empty
	self.selection_changed(None)

    def object_list_callback(self, *args):
	win = self.xygraph.show_plot_objects_list_window()
	self.set_transient_for(win.win)
	win.show()

    def add_object_callback(self, *args):
	self.set_transient_for(self.xygraph.show_add_object_dialog().dialog)

    def zoom_in_callback(self, *args):
	self.xygraph.zoom_in()

    def zoom_out_callback(self, *args):
	self.xygraph.zoom_out()

    def zoom_area_callback(self, uic, path, event_type, state):
	state = util.parse_bool(state)
	self.xygraph.zoom_area(state)

    def grid_toggle_cb(self, uic, path, event_type, state):
	state = util.parse_bool(state)
	self.xygraph.set_grid_active(state)
	gconf.client_get_default().set_bool(GRID_TOGGLE_KEY, state)

    def axis_toggle_cb(self, uic, path, event_type, state):
	state = util.parse_bool(state)
	self.xygraph.set_axis_active(state)
	gconf.client_get_default().set_bool(AXIS_TOGGLE_KEY, state)
	    
    def legend_toggle_cb(self, uic, path, event_type, state):
	state = util.parse_bool(state)
	self.xygraph.set_legend_active(state)
	gconf.client_get_default().set_bool(LEGEND_TOGGLE_KEY, state)

    def refresh_cb(self, ui_component, id):
	self.xygraph.refresh()

    def modified_status_changed(self):
	self.__persist_stream.dirty = self.xygraph.modified
	    
    def print_cb(self, *args):
	print "Print!"
	printing.show_print_dialog(self)

    def selection_changed(self, sel):
	uic = self.get_ui_component()
	if sel is None:
	    sensitive = "0" # "False"
	else:
	    sensitive = "1" # "True"
	uic.set_prop("/commands/EditClear", "sensitive", sensitive)
	uic.set_prop("/commands/EditProperties", "sensitive", sensitive)
	uic.set_prop("/commands/EditCut", "sensitive", sensitive)
	uic.set_prop("/commands/EditCopy", "sensitive", sensitive)
	
    def clear_cb(self, *args):
	self.xygraph.plot_area.clear_selected()

    def edit_properties_cb(self, *args):
	self.xygraph.plot_area.selected_properties()

    def edit_cut_cb(self, *args):
	self.xygraph.plot_area.edit_cut()
    def edit_copy_cb(self, *args):
	self.xygraph.plot_area.edit_copy()
    def edit_paste_cb(self, *args):
	self.xygraph.plot_area.edit_paste()
    def edit_haxis_cb(self, *args):
	self.xygraph.create_haxis_properties_dialog()
    def edit_vaxis_cb(self, *args):
	self.xygraph.create_vaxis_properties_dialog()

