#! /usr/bin/env python

import gtk, gtk.glade as glade
import gnumexp_conf
from registry import *
from lineattr import *
from copy import *
from gettext import gettext as _


class PlotObjectDialog:
    def __init__(self):
	self.xml = glade.XML(gnumexp_conf.GLADEDIR + "/xygraph.glade2",
			     "function dialog")
	self.dialog = self.xml.get_widget("function dialog")

	attr = LineAttr(thickness=5.0, dash_style=Linemode.SOLID, color=Color(0,0,0))
	win = gtk.Window()
	bt = LineAttrButton(attr)
	bt.show()
	self.lineattr_bt = bt
	hbox = self.xml.get_widget("lineattr_container")
	hbox.add(bt)
	self.xml.get_widget("line style label").set_mnemonic_widget(bt)

	hbox = gtk.HBox()
 	opmenu = self.xml.get_widget("func type")
 	menu = gtk.Menu()
	self.contents_list = plotobjectcontents_list_new()
	self.contents = None
	hist = 0
	for contents in self.contents_list:
	    contents.widget.hide()
	    contents.opmenu_history = hist
	    hbox.add(contents.widget)
	    item = gtk.MenuItem(_(contents.name))
	    item.set_data("contents", contents)
	    item.show()
 	    menu.append(item)
	    hist += 1

    	opmenu.set_menu(menu)
	opmenu.connect("changed", self.type_changed)

	self.select_contents(self.contents_list[0])
	self.dialog.vbox.add(hbox)
	hbox.show()
	self.dialog.connect_after("response", self.__dialog_response)
	

    def type_changed(self, opmenu):
	item = opmenu.get_menu().get_active()
	self.select_contents(item.get_data('contents'))

    def select_contents(self, contents):
	assert isinstance(contents, PlotObjectContents)
	if self.contents is not None:
	    self.contents.widget.hide()
	self.contents = contents
	self.contents.widget.show()
	opmenu = self.xml.get_widget("func type")
	opmenu.set_history(contents.opmenu_history)

    def set_type_menu_frozen(self, frozen):
	opmenu = self.xml.get_widget("func type")
	opmenu.set_sensitive(not frozen)

    def __dialog_response(self, dialog, response):
	dialog.destroy()
	self.dialog = None

    def get_values(self):
	label = self.xml.get_widget("func caption").get_text()
	lineattr = deepcopy(self.lineattr_bt.attr)
	type_name = self.contents.name
	rv = { 'label': label,
	       'lineattr': lineattr,
	       'type': type_name,
	       'contents': self.contents.get_values(),
	       }
	return rv

    def set_values(self, values):
	self.xml.get_widget("func caption").set_text(values['label'])
	self.lineattr_bt.set_attr(deepcopy(values['lineattr']))
	type_name = values['type']
	for contents in self.contents_list:
	    if contents.name == type_name:
		contents.set_values(values['contents'])
		self.select_contents(contents)
		break


def __dialog_response(wid, response, dialog):
    if response == gtk.RESPONSE_ACCEPT:
	print "response accept!"
	v = dialog.get_values()
	print v

if __name__ == '__main__':
    gnumexp_conf.GLADEDIR = '.'
    win = PlotObjectDialog()
    win.dialog.connect('destroy', gtk.mainquit)
    win.dialog.show()
    win.dialog.connect("response", __dialog_response, win)
    gtk.mainloop()

