import gobject
import pango
import gtk
import bonobo
import CORBA
import warnings

class ExpressionEntry(gtk.Frame):
    __gproperties__ = { 'variables-list' :
                        (object,
                         'List of variables',
                         'List of variables found in the expression',
                         gobject.PARAM_READABLE)
                        }

    def __init__(self, kernel):
        gtk.Frame.__init__(self)
        gtk.widget_push_composite_child()
        self.set_shadow_type(gtk.SHADOW_IN)
        self.entry = gtk.Entry()
        self.entry.modify_font(pango.FontDescription("Monospace"))
        self.entry.show()
        self.entry.set_has_frame(False)
        self.img = gtk.Image()
        icon_size = gtk.ICON_SIZE_SMALL_TOOLBAR
        self.img.set_from_stock(gtk.STOCK_DIALOG_WARNING, icon_size)
        self.img.set_sensitive(False)
        self.img.show()
        eventbox = gtk.EventBox()
        eventbox.set_visible_window(True)
        eventbox.set_above_child(False)
        eventbox.add(self.img)
        w, h = gtk.icon_size_lookup(icon_size)
        w += 4
        eventbox.set_size_request(w, h)
        eventbox.show()
        self.entry.connect("style-set", self.__entry_style_set, eventbox)
        hbox = gtk.HBox(False, 0)
        hbox.show()
        hbox.pack_start(eventbox, False, False, 0)
        hbox.add(self.entry)
        self.add(hbox)
        self.kernel = kernel
        self.__timeout = None
        self.entry.connect("changed", self.__entry_changed)
        self.__varlist = []
        gtk.widget_pop_composite_child()

    def __set_varlist(self, varlist):
        if varlist != self.__varlist:
            self.__varlist = varlist
            self.notify("variables-list")

    def __validate(self):
        self.__timeout = None
        expr = self.entry.get_text()
        if expr:
            try:
                el = self.kernel.parseString(expr)
            except CORBA.UserException:
                el = None
        else:
            el = None
        if el is None:
            self.img.set_sensitive(True)
        else:
            self.img.set_sensitive(False)
            varlist = []
            for var in self.kernel.elementListVariables(el):
                if var not in varlist:
                    varlist.append(var)
            self.__set_varlist(varlist)
            self.kernel.elementUnref(el)
        return False

    def __entry_changed(self, entry):
        if self.__timeout is None:
            self.__timeout = gobject.timeout_add(100, self.__validate)

    def __entry_style_set(self, entry, prev_style, eventbox):
        for state in (gtk.STATE_NORMAL, gtk.STATE_ACTIVE, gtk.STATE_PRELIGHT,
                      gtk.STATE_SELECTED, gtk.STATE_INSENSITIVE):
            eventbox.modify_bg(state, entry.get_style().base[state])

    def do_get_property(self, property):
        if property.name == 'variables-list':
            return self.__varlist
        else:
            raise AttributeError, 'unknown property %s' % property.name

    def get_text(self):
        return self.entry.get_text()
    def set_text(self, text):
        self.entry.set_text(text)
        self.__validate()
        
        
gobject.type_register(ExpressionEntry)


class VariableEntry(gtk.ComboBox):
    def __init__(self, expr_ent, expr_ent2=None):
        varlist = gtk.ListStore(str)
        gtk.ComboBox.__init__(self, varlist)
        cell = gtk.CellRendererText()
        self.pack_start(cell, True)
        self.add_attribute(cell, 'text', 0)
        self.__entry1 = expr_ent
        self.__entry2 = expr_ent2
        expr_ent.connect("notify::variables-list", self.__update_varlist, varlist)
        if expr_ent2 is not None:
            expr_ent2.connect("notify::variables-list", self.__update_varlist, varlist)

    def __update_varlist(self, entry, param, varlist):
        i = self.get_active_iter()
        if i:
            prev = varlist.get(i, 0)[0]
        else:
            prev = None
        varlist.clear()
        new_sel_iter = None

        list1 = self.__entry1.get_property("variables-list")
        if self.__entry2 is None:
            intersection = list1
        else:
            list2 = self.__entry2.get_property("variables-list")
            intersection = []
            for item in list1:
                if item in list2:
                    intersection.append(item)
        
        for var in intersection:
            i = varlist.append()
            varlist.set(i, 0, var)
            if var == prev:
                new_sel_iter = i
        if new_sel_iter is None:
            new_sel_iter = varlist.iter_nth_child(None, 0)
        if new_sel_iter is None:
            self.set_active(-1)
        else:
            self.set_active_iter(new_sel_iter)
    
    def get_value(self):
        i = self.get_active_iter()
        if i is None:
            return None
        else:
            return self.get_model().get(i, 0)[0]

    def set_value(self, value):
        if value:
            for row in self.get_model():
                if row[0] == value:
                    self.set_active_iter(row.iter)
                    break
            else:
                warnings.warn("Could not find %s in valuelist model" % value)
        

def get_console_entry():
    entry = gtk.Entry()
    entry.modify_font(pango.FontDescription("Monospace"))
    entry.show()
    return entry

def create_indenting_container(title, child):
    vbox = gtk.VBox(); vbox.show()
    vbox.set_spacing(6)
    label = gtk.Label(); label.show()
    label.set_markup('<span weight="bold">%s</span>' % title)
    label.set_alignment(0, 0.5)
    vbox.add(label)
    hbox = gtk.HBox(); hbox.show()
    vbox.add(hbox)
    label = gtk.Label('    '); label.show()
    hbox.add(label)
    hbox.add(child)
    return vbox


if __name__ == '__main__':
    import bonobo
    kernel = bonobo.get_object("OAFIID:Numexp_Kernel", "Bonobo/Unknown")
    w = gtk.Window()
    a = gtk.Alignment(xalign=0.5, yalign=0.5, xscale=1.0, yscale=0.0)
    w.set_border_width(6)
    entry = ExpressionEntry(kernel)
    def __variables_notify(entry, param):
        print entry.get_property("variables-list")
    entry.connect("notify::variables-list", __variables_notify)
    a.add(entry)
    a.show()
    entry.show()
    w.add(a)
    w.show()
    w.connect("destroy", lambda w: gtk.main_quit())
    gtk.main()
    
