#! /usr/bin/env python
from warnings import warn
import gtk
from nxplot import *
from misc import *


class Legend(CanvasObjectBase):
    class DrawData: pass
    
    def __init__(self, plotarea):
	CanvasObjectBase.__init__(self)
	self.bbox = NxplotRect(1, 1, 2, 2)
	self.set_property('interaction-mask', InteractionMask.MOVE)
	self.plotarea = plotarea
	self.parent_bbox_sz = 1		# diagonal of plotarea's bbox
	self.objlist = []		# list of plot objects
	self.set_property('whole-area-interaction', 1)

    def set_objects_list(self, objlist):
	"""Sets list of objects represented in the legend.""
	@objlist: list<PlotObject>"""
	self.objlist = objlist
	self.draw_data_list = None

    def real_configure(self, new_bbox, interaction):
	if interaction == Interaction.MOVE:
	    #print "Legend.real_configure: Interaction.MOVE: %f x %f" % (self.size.x, self.size.y)
	    bbox1 = NxplotRect(new_bbox.x1, new_bbox.y1,
			       new_bbox.x1 + self.size.x,
			       new_bbox.y1 + self.size.y)
	    self.plotarea.legend_geometry_request(bbox1)
	elif interaction == Interaction.MOVE_RESIZE:
	    #print "Legend.real_configure: Interaction.MOVE_RESIZE"
	    self.bbox = NxplotRect(new_bbox.x1, new_bbox.y1,
				   new_bbox.x1 + self.size.x,
				   new_bbox.y1 + self.size.y)
	else:
	    warn("Legend.real_configure: unexpected interaction!")

    def real_draw(self, pl):
	pl.linewidth(self.linewidth)
	pl.pencolor(0,0,0)
	pl.fillcolorname("ivory")
	pl.fillopacity(0xC000)
	pl.box(self.bbox.x1 + self.linewidth/2,
	       self.bbox.y1 + self.linewidth/2,
	       self.bbox.x2 - self.linewidth/2,
	       self.bbox.y2 - self.linewidth/2)
	pl.fillopacity(0)
	pl.translate(self.bbox.x1 + self.linewidth + self.borderwidth,
		     self.bbox.y2 - self.linewidth - self.borderwidth)
	pl.fontsize(self.fontsize)
	pl.fontname(self.fontname)
	maxwidth = 0; maxheight = 0;

	for obj, draw_data in zip(self.objlist, self.draw_data_list):
	    sw, sh = draw_data.sample_width, draw_data.sample_height
	    height = draw_data.height

	    # draw legend label
	    pl.savestate()
	    pl.translate(self.maxsamplewidth + self.sample_spacing,
			 -draw_data.label_ascender)
	    obj.draw_legend_label(pl, self.attributes, 1, draw_data.label_data)
	    pl.restorestate()

	    # draw legend sample
	    pl.savestate()
	    pl.translate(self.maxsamplewidth/2 - sw/2,
			 -height + height/2 - sh/2)
	    obj.draw_legend_sample(pl, 1)
	    pl.restorestate()

	    pl.translate(0, -(height + self.vspacing))

    def _update_size(self):
	pl = self.get_plotter()
	if pl is None:
	    return
	pl.open()
	# some 'constants'
	self.fontsize       = self.parent_bbox_sz*20e-3
	self.borderwidth    = self.parent_bbox_sz*5e-3
	self.vspacing       = self.fontsize*0.5
	self.linewidth      = self.parent_bbox_sz*1e-3
	self.sample_spacing = self.parent_bbox_sz*20e-3
	self.fontname       = "Sans Regular"
	# compute maximum label width/height
	#pl.fontsize(self.fontsize)
	#pl.fontname(self.fontname)
	self.attributes =  { 'fontsize': self.fontsize,
			     'fontname': self.fontname }
	maxwidth = 0; maxheight = 0; maxsamplewidth = 0;
	self.draw_data_list = []

	for obj in self.objlist:
	    draw_data = self.DrawData()
	    self.draw_data_list.append(draw_data)

	    # legend label
	    label_data = obj.draw_legend_label(pl, self.attributes, 0)
	    draw_data.label_width     = label_data[0]
	    draw_data.label_ascender  = label_data[1]
	    draw_data.label_descender = label_data[2]
	    draw_data.label_data      = label_data[3]
	    maxwidth = max(maxwidth, draw_data.label_width)

	    # legend sample
	    sw, sh = obj.draw_legend_sample(pl, 0)
	    draw_data.sample_width  = sw
	    draw_data.sample_height = sh
	    maxsamplewidth = max(maxsamplewidth, sw)

	    draw_data.height = max(draw_data.label_ascender +
				   draw_data.label_descender, sh)
	    maxheight += draw_data.height + self.vspacing
	maxheight -= self.vspacing # last row should not have vspacing

	width  = self.borderwidth*2 + self.linewidth + maxwidth +\
		 maxsamplewidth + self.sample_spacing
	height = self.borderwidth*2 + self.linewidth + maxheight
	self.size = NxplotPoint(width, height)
	self.maxsamplewidth = maxsamplewidth
    
    def real_size_request(self):
	self._update_size()
	return self.size

    def real_enter(self):
	pass #print self, "enter()"

    def real_leave(self):
	pass #print self, "leave()"

