from nxplot import *


class Marker:
    """Abstract base class for Markers"""
    def __init__(self, radius):
	pass
    
    def draw(self, pl, x, y):
	pass

    def __str__(self):
	return self.name


class MarkerX:
    """A 'diagonal cross' type (X) marker"""

    name = 'X'
    
    def __init__(self, radius):
	self.r = radius

    def draw(self, pl, x, y):
	r = self.r
	pl.endpath()
	pl.savestate()
	pl.linemode(Linemode.SOLID)
	pl.line(x - r, y - r, x + r, y + r)
	pl.line(x + r, y - r, x - r, y + r)	
	pl.endpath()
	pl.restorestate()

class MarkerO:
    """A 'circle' type (O) marker"""

    name = 'O'
    
    def __init__(self, radius):
	self.r = radius

    def draw(self, pl, x, y):
	pl.endpath()
	pl.savestate()
	pl.linemode(Linemode.SOLID)
	pl.circle(x, y, self.r*.75)
	pl.endpath()
	pl.restorestate()

class MarkerP:
    """An 'upright cross' type (+) marker"""

    name = '+'
    
    def __init__(self, radius):
	self.r = radius

    def draw(self, pl, x, y):
	r = self.r
	pl.endpath()
	pl.savestate()
	pl.linemode(Linemode.SOLID)
	pl.line(x, y - r, x, y + r)
	pl.line(x - r, y, x + r, y)	
	pl.endpath()
	pl.restorestate()

class MarkerB:
    """A 'box' type (square) marker"""

    name = unichr(0x025A1)
    
    def __init__(self, radius):
	self.r = radius

    def draw(self, pl, x, y):
	r = self.r*.75
	pl.savestate()
	pl.linemode(Linemode.SOLID)
	pl.box(x - r, y - r, x + r, y + r)
	pl.restorestate()


all_markers = [
    MarkerX,
    MarkerO,
    MarkerP,
    MarkerB,
    ]
    
