from misc import *
from kernel import *
from plotobject_registry import *
from dialogs.registry import *
from functionobject import *
import dialogs.util
import xml.dom.minidom as dom
import types, traceback
import ORBit, CORBA
import xyconsts
from gettext import gettext as _
def N_(message): return message


class Parametric(Function):

    def __init__(self, values, plotarea):
        self.expression_x = None
        self.expression_y = None
        self.variable     = None
        self.range_start  = None
        self.range_end    = None
	self.bbox         = None
	self.samples      = None
	self.mathml       = '<math><mtext></mtext></math>'
	self.nsamples     = 200

        PlotObject.__init__(self)
	self.caption = "%e"

	# keep this last!
	if type(values) is DictionaryType:
	    self.set_values(values)
	elif isinstance(values, dom.Element):
	    self.from_xml(values, plotarea)
	else:
	    raise TypeError("expected dictionary or DOM Element")

        self.set_state(self.STATE_NORMAL)

    def get_values(self):
	values = PlotObject.get_values(self)
	values['contents'] = { 'expression-x': 		self.expression_x,
			       'expression-y': 		self.expression_y,
			       'independent-variable': 	self.variable,
			       'range-start': 		self.range_start,
			       'range-end': 		self.range_end,
			       'nsamples': 		self.nsamples,
			       }
	values['type'] = 'Parametric'
	return values

    def set_values(self, values):
	PlotObject.set_values(self, values)
	values = values['contents']
	self.expression_x = values['expression-x']
	self.expression_y = values['expression-y']
	self.variable     = values['independent-variable']
	self.range_start  = values['range-start']
	self.range_end    = values['range-end']
	self.nsamples     = values['nsamples']
    	self.update_function()
	self.__update_mathml()

    def __update_mathml(self):
	def expression_to_mathml(expression, name):
	    try:
		el = kernel.parseString(expression)
		expression_mathml = kernel.elementToMathML(el)
		kernel.elementUnref(el)
	    except:
		expression_mathml = '<merror>error converting %s'\
				    ' to MathML</merror>' % name
	    return expression_mathml

	expression_x_mathml = expression_to_mathml(self.expression_x, "expression X")
	expression_y_mathml = expression_to_mathml(self.expression_y, "expression Y")
	variable_mathml = '<mi>' + self.variable + '</mi>'
	self.mathml = self._label_to_mathml(self.caption,
					    {'%x': expression_x_mathml,
					     '%y': expression_y_mathml,
					     '%v': variable_mathml})


    def sample(self):
	if (self.expression_x is None or
	    self.expression_y is None): return
	nsamples = self.nsamples
	yvalues = numexp_kernel_linear_sample(self.expression_y, self.variable,
                                              self.range_start, self.range_end, nsamples)
	xvalues = numexp_kernel_linear_sample(self.expression_x, self.variable,
                                              self.range_start, self.range_end, nsamples)
	# Combine X and Y values and store the result
	self.samples = zip(xvalues, yvalues)



    def to_xml(self, doc):
	assert isinstance(doc, dom.Document)
	elem = PlotObject.to_xml(self, doc)
	elem.setAttribute("type", "Parametric")
	
	expr = doc.createElement("expression_x")
	expr.appendChild(doc.createTextNode(self.expression_x))
	elem.appendChild(expr)

	expr = doc.createElement("expression_y")
	expr.appendChild(doc.createTextNode(self.expression_y))
	elem.appendChild(expr)

	var = doc.createElement("variable")
	var.setAttribute("name", self.variable)
	elem.appendChild(var)

	samp = doc.createElement("sampling")
	samp.setAttribute("start", self.range_start)
	samp.setAttribute("end", self.range_end)
	samp.setAttribute("nsamples", str(self.nsamples))
	elem.appendChild(samp)

	return elem

    def from_xml(self, elem, plotarea):
	PlotObject.from_xml(self, elem, plotarea)
	assert elem.getAttribute("type") == 'Parametric'

	expr = elem.getElementsByTagName('expression_x')[0]
	self.expression_x = ""
	for node in expr.childNodes:
	    if node.nodeType == node.TEXT_NODE:
		self.expression_x += node.data
	self.expression_x = self.expression_x.strip().encode()
	
	expr = elem.getElementsByTagName('expression_y')[0]
	self.expression_y = ""
	for node in expr.childNodes:
	    if node.nodeType == node.TEXT_NODE:
		self.expression_y += node.data
	self.expression_y = self.expression_y.strip().encode()

	var = elem.getElementsByTagName('variable')[0]
	self.variable = var.getAttribute("name").encode()

	samp = elem.getElementsByTagName('sampling')[0]
	self.range_start = samp.getAttribute("start").encode()
	self.range_end = samp.getAttribute("end").encode()
	self.nsamples = int(samp.getAttribute("nsamples"))
	
	self.__update_mathml()

	

class ParametricContents(PlotObjectContents):
    def __init__(self):
	PlotObjectContents.__init__(self, "func contents parametric")
	self.name = "Parametric"

	self.expression_x_wid = dialogs.util.ExpressionEntry(kernel)
        self.expression_x_wid.show()
	self.xml.get_widget("expression X container").add(self.expression_x_wid)
	self.xml.get_widget("expression X label").set_mnemonic_widget(self.expression_x_wid)

	self.expression_y_wid = dialogs.util.ExpressionEntry(kernel)
        self.expression_y_wid.show()
	self.xml.get_widget("expression Y container").add(self.expression_y_wid)
	self.xml.get_widget("expression Y label").set_mnemonic_widget(self.expression_y_wid)
	
	self.independent_var_wid = dialogs.util.VariableEntry(self.expression_x_wid,
                                                              self.expression_y_wid)
        self.independent_var_wid.show()
	self.xml.get_widget("variable container").add(self.independent_var_wid)
	self.xml.get_widget("variable label").set_mnemonic_widget(self.independent_var_wid)

	self.range_start_wid     = self.xml.get_widget("range start")
	self.range_end_wid       = self.xml.get_widget("range end")
	self.nsamples_wid        = self.xml.get_widget("nsamples")

    def get_values(self):
	rv = {'expression-x': 		self.expression_x_wid.get_text(),
	      'expression-y': 		self.expression_y_wid.get_text(),
	      'independent-variable': 	self.independent_var_wid.get_value(),
	      'range-start':		self.range_start_wid.get_text(),
	      'range-end':		self.range_end_wid.get_text(),
	      'nsamples':		int(self.nsamples_wid.get_text()),
	      }
	return rv

    def set_values(self, values):
	self.expression_x_wid.set_text(values['expression-x'])
	self.expression_y_wid.set_text(values['expression-y'])
	self.independent_var_wid.set_value(values['independent-variable'])
	self.range_start_wid.set_text(values['range-start'])
	self.range_end_wid.set_text(values['range-end'])
	self.nsamples_wid.set_text(str(values['nsamples']))



plotobjectcontents_list.append(ParametricContents)
type_name_to_class_mapping[N_('Parametric')] = Parametric
__all__ = ['Parametric']
print "'Parametric' plot object has been registered"

