from __future__ import division
import pygtk; pygtk.require("2.0")
import bonobo, Bonobo, Bonobo__POA
import bonobo.ui
import gnomevfs
import gtk
import gobject
from xml.sax.saxutils import escape

class PersistStream(Bonobo__POA.PersistStream, bonobo.UnknownBaseImpl):
    def __init__(self, load_cb, save_cb):
        bonobo.UnknownBaseImpl.__init__(self)
	self.load_cb            = load_cb
	self.save_cb            = save_cb
	self.dirty              = False
	self.content_types      = []
        self.iid                = None

    # implementations of methods/attributes of "Interface" go here
    def load(self, stream, content_type):
	#print "PersistStream::load(content_type='%s')" % content_type
	self.load_cb(stream, content_type)

    def save(self, stream, content_type):
	#print "PersistStream::save(content_type='%s')" % content_type
	self.save_cb(stream, content_type)

    def add_content_type(self, content_type):
	self.content_types.append(content_type)

    def getContentTypes(self):
	"""Gets a list of supported mime types that this persistor can
	save its data in. The default / preferred type being the first
	element.  """
	return self.content_types

    def getIId(self):
	"""Get the IID necessary to recreate the object whose internal
	state this interface represents"""
        return self.iid

    def isDirty(self):
	return self.dirty


class VFSAsyncBaseGUI(object):
    """Do _NOT_ transfer large files with this class!!!"""
    def __init__(self, parent):
        self.dialog = None
        self.__parent = parent
        self.stage_label = None
        self.gui_show_timeout = None
        self.progress_bar = None
        self.current_progress = None
        self.current_status = None
        self.__gui_update_timeout = None
        self.__gui_show_timeout = None
        self.handle = None
        self.status_label = None
        self.aborted = False
        self._abort_exception = None
        
    def set_status(self, status):
        self.current_status = status
        print "** STATUS ** ", status
        if self.progress_bar is not None:
            self.__update_gui_progress_status()

    def set_progress(self, progress):
        self.current_progress = progress
        if self.progress_bar is not None:
            if self.__gui_update_timeout is None:
                self.__gui_update_timeout = gobject.timeout_add \
                                            (100, self.__update_gui_progress_status)

    def __update_gui_progress_status(self):
        self.__gui_update_timeout = None
        if self.dialog is None:
            return
        self.status_label.set_text(self.current_status)
        if self.current_progress is None:
            self.progress_bar.pulse()
        else:
            self.progress_bar.set_fraction(self.current_progress)

    def _shutdown_gui(self):
        if self.__gui_update_timeout is not None:
            gobject.source_remove(self.__gui_update_timeout)
            self.__gui_update_timeout = None
        if self.__gui_show_timeout is not None:
            gobject.source_remove(self.__gui_show_timeout)
            self.__gui_show_timeout = None
        if self.dialog is not None:
            self.dialog.destroy()
            self.dialog = None

    def abort(self, exception):
        self.aborted = True
        self._abort_exception = exception
        if exception is None:
            self.set_status("Aborted")
        else:
            self.set_status("Error")
        if self.handle is not None:
            self.handle.cancel()
            self.handle = None
        bonobo.main_quit()

    def _show_gui(self, timeout=1000):
        if self.dialog is not None:
            return
        if timeout is None:
            if self.__gui_show_timeout is not None:
                gobject.source_remove(self.__gui_show_timeout)
                self.__gui_show_timeout = None
            self.__show_gui_timeout()
        else:
            if self.__gui_update_timeout is None:
                self.__gui_update_timeout = gobject.timeout_add(1000, self.__show_gui_timeout)

    def __show_gui_timeout(self):
        self.__gui_update_timeout = None
        self.dialog = gtk.Dialog(title="Loading...",
                                 buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
        self.dialog.set_border_width(12)
        self.dialog.set_has_separator(False)
        self.dialog.vbox.set_spacing(8)
        if self.__parent is not None:
            if isinstance(self.__parent, bonobo.ui.Control):
                self.__parent.set_transient_for(self.dialog)
            else:
                self.dialog.set_transient_for(self.__parent)
        #vfsuri = gnomevfs.URI(self.uri)
        label = gtk.Label("")
        label.set_markup("<big><b>Loading document</b></big>\n"
                         "<tt>%s</tt>" % escape(self.uri))
        self.dialog.vbox.add(label)
        self.progress_bar = gtk.ProgressBar()
        self.dialog.vbox.add(self.progress_bar)

        self.status_label = gtk.Label()
        self.dialog.vbox.add(self.status_label)        

        self.__update_gui_progress_status()
        self.dialog.show_all()
        self.dialog.connect("response", self.__dialog_response)
        
    def __dialog_response(self, dialog, response):
        if response == gtk.RESPONSE_REJECT or \
           response == gtk.RESPONSE_DELETE_EVENT:
            self.abort(None)


class VFSAsyncLoaderGUI(VFSAsyncBaseGUI):
    """Do _NOT_ load large files with this class!!!"""
    def __init__(self, parent=None):
        VFSAsyncBaseGUI.__init__(self, parent)
        self.buffer = []
        self.bytes_read = 0
        self.size = None
        self.block_size = None
        
    def load(self, uri):
        self.uri = uri
        gnomevfs.async.get_file_info(uri, self.__info_cb)
        self.set_status("Obtaining information...")
        self._show_gui()
        bonobo.main()
        self._shutdown_gui()
        self.handle = None
        if self.aborted:
            if self._abort_exception is None:
                raise gnomevfs.Error
            else:
                raise self._abort_exception
        data = "".join(self.buffer)
        self.buffer = None
        return data

    def __info_cb(self, handle, results):
        if self.aborted:
            return
        assert len(results) == 1
        uri, exception, info = results[0]
        if exception is None:
            try:
                self.size = info.size
            except ValueError:
                pass
        if self.size is None:
            self.block_size = 1024
        else:
            self.block_size = min(1024, self.size // 20)
        self.set_status("Opening...")
        gnomevfs.async.open(self.uri, self.__open_cb)

    def __open_cb(self, handle, exception):
        if self.aborted:
            return
        if exception is not None:
            self.abort(exception)
            return
        self.handle = handle
        handle.read(self.block_size, self.__read_cb)
        self.set_status("Reading...")

    def __close_cb(self, handle, exception):
        self.set_status("Closed")
        self.handle = None
        bonobo.main_quit()

    def __read_cb(self, handle, buffer, exception, requested):
        if self.aborted:
            return
        if exception is not None:
            print "read exception", exception
            if issubclass(exception, gnomevfs.EOFError):
                handle.close(self.__close_cb)
                self.handle = None
                bonobo.main_quit()
                return
            else:
                self.abort(exception)
                return
        #print "requested %i got %i" % (requested, len(buffer))
        self.buffer.append(buffer)
        self.bytes_read += len(buffer)
        if self.size is None:
            self.set_progress(None)
        else:
            self.set_progress(self.bytes_read / self.size)
        if self.size is not None:
            if self.bytes_read >= self.size:
                if self.handle is not None:
                    handle.close(self.__close_cb)
                    self.handle = None
            else:
                handle.read(self.block_size, self.__read_cb)
        else:
            handle.read(self.block_size, self.__read_cb)
        
class VFSAsyncSaverGUI(VFSAsyncBaseGUI):
    """Do _NOT_ save large files with this class!!!"""
    def __init__(self, parent=None):
        VFSAsyncBaseGUI.__init__(self, parent)
        self.buffer = None
        self.bytes_written = 0
        self.block_size = None
        
    def save(self, uri, buffer, confirm_overwrite=True):
        self.uri = uri
        self.buffer = buffer
        self.block_size = max(1024, len(self.buffer) // 20)
        if confirm_overwrite:
            self.set_status("Obtaining information...")
            gnomevfs.async.get_file_info(uri, self.__info_cb)
        else:
            self.set_status("Creating file...")
            gnomevfs.async.create(self.uri, self.__create_cb)
        self._show_gui()
        bonobo.main()
        self._shutdown_gui()
        self.buffer = None
        self.handle = None
        if self.aborted:
            if self._abort_exception is None:
                raise gnomevfs.Error
            else:
                raise self._abort_exception

    def __info_cb(self, handle, results):
        if self.aborted:
            return
        assert len(results) == 1
        uri, exception, info = results[0]
        uri_exists = True
        if exception is not None:
            print "exception:::::::::::: ", exception
            if issubclass(exception, gnomevfs.IOError):
                uri_exists = False
            else:
                self.abort(exception)
        if uri_exists:
            self._show_gui(timeout=None)
            dialog = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION, parent=self.dialog)
            dialog.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                               gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT)
            dialog.set_markup(("<big><b>Overwrite file?</b></big>\n\n"
                               "The following file already exists. "
                               " Press Save to overwrite it.\n"
                               "\n<tt>%s</tt>") % self.uri)
            response = dialog.run()
            dialog.destroy()
            if response != gtk.RESPONSE_ACCEPT:
                self.abort(None)
                return
        gnomevfs.async.create(self.uri, self.__create_cb)

    def __create_cb(self, handle, exception):
        if self.aborted:
            return
        if exception is not None:
            self.abort(exception)
            return
        self.handle = handle
        handle.write(self.buffer[0 : min(self.block_size, len(self.buffer))], self.__write_cb)
        self.set_status("Writing...")

    def __close_cb(self, handle, exception):
        self.set_status("Closed")
        self.handle = None
        bonobo.main_quit()

    def __write_cb(self, handle, bytes_written, exception, bytes_requested):
        if self.aborted:
            return
        if exception is not None:
            print "write exception", exception
            self.abort(exception)
            return
        self.bytes_written += bytes_written
        self.set_progress(self.bytes_written / len(self.buffer))
        if self.bytes_written >= len(self.buffer):
            if self.handle is not None:
                self.set_status("Closing...")
                handle.close(self.__close_cb)
                self.handle = None
        else:
            handle.write(self.buffer[self.bytes_written :
                                     min(self.bytes_written + self.block_size,
                                         len(self.buffer))],
                         self.__write_cb)



class PersistURI(Bonobo__POA.PersistFile, bonobo.UnknownBaseImpl):
    def __init__(self, load_cb, save_cb, parent):
        bonobo.UnknownBaseImpl.__init__(self)
	self.load_cb = load_cb
	self.save_cb = save_cb
	self.dirty = False
        self.iid = None
        self.uri = None
        self.content_types = []
        self.__parent = parent

    # raises (IOError, NotSupported, WrongDataType, FileNotFound);
    def load(self, uri):
        loader = VFSAsyncLoaderGUI(parent=self.__parent)
        try:
            data = loader.load(uri)
        except:
            raise Bonobo.IOError
        try:
            self.load_cb(data)
        except:
            raise Bonobo.Persist.WrongDataType
        self.uri = uri
        self.dirty = False

    # raises (IOError, NotSupported);
    def save(self, uri):
	contents = self.save_cb()
        saver = VFSAsyncSaverGUI(parent=self.__parent)
        try:
            saver.save(uri, contents, False)
        except gnomevfs.Error, ex:
            print "save exception: %s" % str(ex.__class__)
            raise Bonobo.IOError
	self.dirty = False
        self.uri = uri

    def getCurrentFile(self):
        if self.uri is None:
            raise Bonobo.PersistFile.NoCurrentName
        return self.uri

    def getContentTypes(self):
	"""Gets a list of supported mime types that this persistor can
	save its data in. The default / preferred type being the first
	element.  """
	return self.content_types

    def getIId(self):
	"""Get the IID necessary to recreate the object whose internal
	state this interface represents"""
        return self.iid

    def isDirty(self):
	return self.dirty

if __name__ == '__main__':
    import sys

    if 0:
        loader = VFSAsyncLoaderGUI()
        data = loader.load(sys.argv[1])
        if data is not None:
            print "SUCCESS!"
            file("foo.out", "w").write(data)
        else:
            print "LOAD FAILED"
    if 1:
        saver = VFSAsyncSaverGUI()
        saver.save(sys.argv[2], file(sys.argv[1]).read(), False)

