#! /usr/bin/env python
import gtk
import gtk.gdk
import gobject
from nxplot import *
from misc import *
from nxpruler import *
from copy import *
from dialogs.objectlist import *
from plotobject_registry import *
import math, types
import xml.dom.minidom as dom
import legend
import axis
import xyconsts
import gnumexp_conf
from gettext import gettext as _
import xygraphC
import weakref


_predefined_lineattrs = [
    LineAttr(thickness=5.0, dash_style=Linemode.SOLID, color=Color(0,0,0)),
    LineAttr(thickness=5.0, dash_style=Linemode.SOLID, color=Color(0,0x8000,0)),
    LineAttr(thickness=5.0, dash_style=Linemode.SOLID, color=Color(0x8000,0,0)),
    LineAttr(thickness=5.0, dash_style=Linemode.SOLID, color=Color(0,0x8000,0x8000)),
    ]

_SELECTION_XYGRAPH_TARGET = gtk.gdk.atom_intern("application/x-xygraph", True)
_SELECTION_CLIPBOARD = gtk.gdk.atom_intern("CLIPBOARD", True)
_SELECTION_TYPE_SRING = gtk.gdk.atom_intern("STRING", True)


class ZoomAreaOverlay(NxpCanvasOverlay):
    def __init__(self):
	NxpCanvasOverlay.__init__(self)
	self.connect("draw", self.__class__.real_draw)

    def real_draw(self, drawable, gc, t):
	gc.set_line_attributes(0, gtk.gdk.LINE_ON_OFF_DASH,
			       gtk.gdk.CAP_NOT_LAST, gtk.gdk.JOIN_MITER)
	drawable.draw_rectangle(gc, 0, self.bbox.x, self.bbox.y,
				self.bbox.width,
				self.bbox.height)


class _ObjectList:
    class _MyModel(PlotObjectListModel):
	def __init__(self, objlist):
	    assert isinstance(objlist, _ObjectList)
	    self.objlist = objlist
	    PlotObjectListModel.__init__(self)

	def row_changed(self, iter, values, obj):
	    """Called when a row has been changed"""
	    obj.set_values(values)
	    self.objlist.object_modified(obj)

	def row_added(self, iter, values):
	    """Called when a row has been added. Should return the
	    user_data argument to associate with the new row."""
	    obj = type_name_to_class_mapping[values['type']](values, None)
	    self.objlist.object_added(obj)
	    return obj

	def row_deleted(self, values, obj):
	    self.objlist.object_deleted(obj)
	    
	def row_selected(self, values, obj):
	    self.objlist.object_selected(obj)


    def __init__(self, plotarea):
	assert isinstance(plotarea, PlotArea)
	self.plotarea     = plotarea
	self.plot_objects = []
	self.model        = self._MyModel(self)

    def object_deleted(self, object):
	self.plot_objects.remove(object)
	self.plotarea.plot_object_deleted(object)
	
    def object_added(self, object):
	self.plot_objects.append(object)
	self.plotarea.plot_object_added(object)

    def add_object(self, object):
	self.plot_objects.append(object)
	self.model.add_object(object.get_values(), object)
	self.plotarea.plot_object_added(object)

    def object_modified(self, object):
	self.plotarea.plot_object_modified(object)

    def show_properties_dialog(self, obj):
	# get iterator for row containing obj
	iter = self.model.find(obj)
	assert iter is not None
	win = self.model.create_properties_dialog(iter)
	return win

    def object_selected(self, object):
	self.plotarea.select_object(object)

    def select_object(self, obj):
	self.model.select_row(self.model.find(obj))

    def delete_object(self, obj):
	self.model.remove_object(self.model.find(obj))


class PlotArea(CanvasContainerBase):

    Ticks = axis.Ticks		# for backward compatibility

    __gsignals__ = {
	"selection-changed": (gobject.SIGNAL_RUN_LAST, # signal flags
			      None, # return type
			      (gobject.TYPE_OBJECT,)) # param types
	}

    def __init__(self, view):
	CanvasContainerBase.__init__(self)
	self.plotobjectlist = None
	self.view = view
	self.clear()
	self.range = NxplotRect(0, 0, 0, 0)
	self.hmajor_attr = LineAttr(color=Color("#ADD8E6"), thickness=2,
				    dash_style=Linemode.SOLID)
	self.vmajor_attr = deepcopy(self.hmajor_attr)
	self.axis_attr = LineAttr(color=Color("blue"), thickness=5,
				  dash_style=Linemode.SOLID)
	self.vticks = self.Ticks(0, 0, 0, 0)
	self.hticks = self.Ticks(0, 0, 0, 0)
	self.vticks_enabled = 0
	self.hticks_enabled = 0
	self.bbox = NxplotRect(0, 0, 0, 0)
	self.bbox_sz = 1
	self.range2bbox_sx = 1
	self.range2bbox_sy = 1
        self.select_area_mode = 0
        self.selecting_area   = 0
        self.overlay = ZoomAreaOverlay()
	self.preselected_object = None
	self.selected_object = None
	
	self.legend = legend.Legend(self)
	self.legend.set_container(self)

	# self.hticks with coordinates translated from function to plot units
	self.hticks_plunit = None
	# self.vticks with coordinates translated from function to plot units
	self.vticks_plunit = None

	self._set_drawable(1)
	self._set_visible(1)
	self.set_property("interaction-mask", 0)

	# horizontal axis
	self.haxis = axis.HorizAxis(pos=0, attr=self.axis_attr,
				    width_scale=xyconsts.WIDTH_SCALE)
	self.haxis.set_container(self)
	# vertical axis
	self.vaxis = axis.VertAxis(pos=0, attr=deepcopy(self.axis_attr),
				    width_scale=xyconsts.WIDTH_SCALE)
	self.vaxis.set_container(self)

	# cross-references: these are needed to handle axis
	# intersection (not drawing tick marks near intersection,
	# which would look ugly)
	self.haxis.vaxis = self.vaxis
	self.vaxis.haxis = self.haxis
	self.connect("notify::canvas", self.__notify_canvas)
	   

    def __notify_canvas(self, *args):
	self.overlay.set_canvas(self.canvas)
	self.canvas.connect("selection-received",
			    self.__selection_received_cb)
	self.canvas.selection_add_target(_SELECTION_CLIPBOARD,
					 _SELECTION_XYGRAPH_TARGET, 0)
	self.canvas.connect("selection-get",
			    self.__selection_get_cb)

    def set_transient_for(self, win):
	self.view.set_transient_for(win)

    def set_haxis_enabled(self, enabled):
	self.haxis_enabled  = enabled
	self.haxis._set_visible(enabled)

    def set_vaxis_enabled(self, enabled):
	self.vaxis._set_visible(enabled)

    def set_legend_enabled(self, enabled):
	self.legend._set_visible(enabled)
    def get_legend_enabled(self):
	return self.legend.visible

    def clear(self):
	"""Clears list of plot objects"""
	if self.plotobjectlist is not None:
	    for obj in self.plotobjectlist.plot_objects:
		self.canvas.remove_object(obj)
	self.plotobjectlist = _ObjectList(self)


    class MyPlotObjectListWindow(PlotObjectListWindow):
	def __init__(self, model, plotarea):
	    PlotObjectListWindow.__init__(self, model)
	    self.plotarea = plotarea
	def create_new_plot_object_dialog(self):
	    win = PlotObjectListWindow.create_new_plot_object_dialog(self)
	    self.plotarea.provide_new_plot_object_defaults(win)
	    return win

    def provide_new_plot_object_defaults(self, win):
	values = win.get_values()
	values['lineattr'] = self.suggest_lineattr()
	win.set_values(values)

    def show_plot_objects_list_window(self):
	win = self.MyPlotObjectListWindow(self.plotobjectlist.model, self)
	return win

    def plot_object_deleted(self, obj):
	self.canvas.remove_object(obj)
	self.view.set_modified(1)
        self.update_legend()

    def delete_plot_object(self, obj):
	if self.selected_object is obj:
	    self.select_object(None)
	if self.preselected_object is obj:
	    self.preselect_object(None)
	self.plotobjectlist.delete_object(obj)
	self.view.set_modified(1)
        self.update_legend()

    def plot_object_modified(self, obj):
	self.update_legend()
	self.view.set_modified(1)

    def plot_object_added(self, obj):
	assert isinstance(obj, PlotObject)
	obj.view = self.view
	obj.set_container(self)
	obj.set_range(self.range.x1, self.range.y1,
		      self.range.x2, self.range.y2)
	self.view.set_modified(1)
	self.update_legend()

    def add_plot_object(self, obj):
	assert isinstance(obj, PlotObject)
	self.plotobjectlist.add_object(obj)

    def real_size_request(self):
	return NxplotPoint(0, 0)

    def real_configure(self, bbox, interaction):
	if self.canvas is None: return
	#print "PlotArea::configure: ", (bbox.x1,bbox.y1,bbox.x2,bbox.y2)
	if interaction != Interaction.MOVE_RESIZE: return
	self.bbox = NxplotRect(bbox.x1, bbox.y1, bbox.x2, bbox.y2)
	for obj in self.get_plot_objects():
	    obj.configure(bbox, interaction)
	self._update_range2bbox()
	self.bbox_sz = math.hypot(bbox.x2 - bbox.x1, bbox.y2 - bbox.y1)
	self.__configure_legend()
	self.__configure_haxis()
	self.__configure_vaxis()

    # <<<axis interface>>>
    def axis_geometry_update_request(self, axis):
	x1, x2, y1, y2 = axis.geometry_request()
	if axis is self.haxis:
	    axis.configure(NxplotRect(self.bbox.x1, y1, self.bbox.x2, y2),
			   Interaction.MOVE_RESIZE)
	elif axis is self.vaxis:
	    axis.configure(NxplotRect(x1, self.bbox.y1, x2, self.bbox.y2),
			   Interaction.MOVE_RESIZE)
	else: assert 0

    def __configure_haxis(self):
	x1, x2, y1, y2 = self.haxis.geometry_request()
	self.haxis.configure(NxplotRect(self.bbox.x1, y1, self.bbox.x2, y2),
			     Interaction.MOVE_RESIZE)

    def __configure_vaxis(self):
	x1, x2, y1, y2 = self.vaxis.geometry_request()
	self.vaxis.configure(NxplotRect(x1, self.bbox.y1, x2, self.bbox.y2),
			     Interaction.MOVE_RESIZE)

    def haxis_position_changed(self, new_pos):
	self.haxis.pos_fn = self.bbox2range_y(new_pos)

    def vaxis_position_changed(self, new_pos):
	self.vaxis.pos_fn = self.bbox2range_x(new_pos)

    def _update_haxis_pos(self):
	self.haxis.set_pos(self.range2bbox_y(self.haxis.pos_fn))
	self.__configure_haxis()

    def _update_vaxis_pos(self):
	self.vaxis.set_pos(self.range2bbox_x(self.vaxis.pos_fn))
	self.__configure_vaxis()

    def set_vaxis_position(self, pos_fn):
	self.vaxis.pos_fn = pos_fn
	self._update_vaxis_pos()

    def set_haxis_position(self, pos_fn):
	self.haxis.pos_fn = pos_fn
	self._update_haxis_pos()
	
    # <<<legend interface>>>
    def legend_geometry_request(self, bbox):
	# we just give the legend whatever geometry it asks for.
	self.legend.configure(NxplotRect(bbox.x1, bbox.y1, bbox.x2, bbox.y2),
			      Interaction.MOVE_RESIZE)

    def __configure_legend(self):
	lbb = self.legend.bbox
	lx, ly = lbb.x1, lbb.y1
	self.legend.parent_bbox_sz = self.bbox_sz
	legendsize = self.legend.size_request()
	if legendsize is None:
	    return
	lw, lh = legendsize.x, legendsize.y
	self.legend.configure(NxplotRect(lx, ly, lx + lw, ly + lh), Interaction.MOVE_RESIZE)

    def update_legend(self):
	l = self.get_plot_objects()
	self.legend.set_objects_list(l)
	self.__configure_legend()

    # <<<>>>
    def set_range(self, x1, y1, x2, y2):
	assert x2 > x1
	assert y2 > y1
	self.range = NxplotRect(x1, y1, x2, y2)
	for obj in self.get_plot_objects():
	    obj.set_range(x1, y1, x2, y2)
	self._update_range2bbox()
	self.queue_draw()

    def get_plot_objects(self):
	return tuple(self.plotobjectlist.plot_objects)
	
    def _update_range2bbox(self):
	if self.bbox is None: return
	try:
	    self.range2bbox_sx = float(self.bbox.x2  - self.bbox.x1) / \
				 float(self.range.x2 - self.range.x1)
	    self.range2bbox_sy = float(self.bbox.y2  - self.bbox.y1) / \
				 float(self.range.y2 - self.range.y1)
	    self._update_ticks_plunit()
	except ZeroDivisionError: pass
	self._update_haxis_pos()
	self._update_vaxis_pos()

    # <<<Conversion from function to canvas units>>>
    def range2bbox_x(self, x):
	return (x - self.range.x1) * self.range2bbox_sx + self.bbox.x1
    def range2bbox_y(self, y):
	return (y - self.range.y1) * self.range2bbox_sy + self.bbox.y1
    # <<<Conversion from canvas to function units>>>
    def bbox2range_x(self, x):
	return (x - self.bbox.x1) / self.range2bbox_sx + self.range.x1
    def bbox2range_y(self, y):
	return (y - self.bbox.y1) / self.range2bbox_sy + self.range.y1


    def _update_ticks_plunit(self):
	if self.hticks is not None:
	    self.hticks_plunit = self.Ticks(
		start  = self.range2bbox_x(self.hticks.start),
		limit  = self.range2bbox_x(self.hticks.limit),
		div    = self.range2bbox_sx * self.hticks.div,
		subdiv = self.range2bbox_sx * self.hticks.subdiv)
	if self.vticks is not None:
	    self.vticks_plunit = self.Ticks(
		start  = self.range2bbox_y(self.vticks.start),
		limit  = self.range2bbox_y(self.vticks.limit),
		div    = self.range2bbox_sy * self.vticks.div,
		subdiv = self.range2bbox_sy * self.vticks.subdiv)
	
    def set_vticks(self, start, div, subdiv, limit):
	self.vticks = self.Ticks(start, div, subdiv, limit)
	self._update_ticks_plunit()
	# vaxis update
	self.vaxis.ticks = self.vticks
	self.vaxis.ticks_plunit = self.vticks_plunit
	self.vaxis.queue_draw()

    def set_hticks(self, start, div, subdiv, limit):
	self.hticks = self.Ticks(start, div, subdiv, limit)
	self._update_ticks_plunit()
	# haxis update
	self.haxis.ticks = self.hticks
	self.haxis.ticks_plunit = self.hticks_plunit
	self.haxis.queue_draw()

    # << drawing >>
    def real_draw(self, pl):
	if self.vticks_enabled and self.vticks is not None:
	    self.draw_vticks(pl)
	if self.hticks_enabled and self.hticks is not None:
	    self.draw_hticks(pl)

    def draw_vticks(self, pl):
	start  = self.vticks_plunit.start
	div    = self.vticks_plunit.div
	subdiv = self.vticks_plunit.subdiv
	limit  = self.vticks_plunit.limit
	x1 = self.bbox.x1
	x2 = self.bbox.x2
	y = start - div
	while y < limit:
	    if y >= start:
		self.vmajor_attr.apply_to(pl, xyconsts.WIDTH_SCALE)
		pl.line(x1, y, x2, y)
	    y += div

    def draw_hticks(self, pl):
	start  = self.hticks_plunit.start
	div    = self.hticks_plunit.div
	subdiv = self.hticks_plunit.subdiv
	limit  = self.hticks_plunit.limit
	y1 = self.bbox.y1
	y2 = self.bbox.y2
	x = start - div
	while x < limit:
	    if x >= start:
		self.hmajor_attr.apply_to(pl, xyconsts.WIDTH_SCALE)
		pl.line(x, y1, x, y2)
	    x += div

    # <<NxpCanvasContainer methods>>
    def real_get_children(self):
	return NxplotList([self.legend] +
			  list(self.get_plot_objects()) +
			  [self.haxis, self.vaxis])

    def _get_child_at_point_helper(self, x, y, children):
	for child in children:
	    if not child.visible: continue
	    x1 = child.wbbox.x
	    x2 = x1 + child.wbbox.width
	    y1 = child.wbbox.y
	    y2 = y1 + child.wbbox.height
	    if x1 <= x < x2 and y1 <= y < y2:
		return child
	return None
	

    def real_get_child_at_point(self, x, y):
	children = (self.legend, self.haxis, self.vaxis)
	child = self._get_child_at_point_helper(x, y, children)
	if child is None:
	    child = self
	return child

    # << XML load/save >>
    def to_xml(self, doc):
	elem = doc.createElement("plotarea")
	elem.setAttribute("x1", str(self.range.x1))
	elem.setAttribute("x2", str(self.range.x2))
	elem.setAttribute("y1", str(self.range.y1))
	elem.setAttribute("y2", str(self.range.y2))
	for obj in self.get_plot_objects():
	    elem1 = obj.to_xml(doc)
	    elem.appendChild(elem1)
	elem.appendChild(self.vaxis.to_xml(doc))
	elem.appendChild(self.haxis.to_xml(doc))
	return elem

    def from_xml(self, elem, replace=1):
	assert isinstance(elem, dom.Element)
	assert elem.tagName == 'plotarea'
	if replace:
	    self.clear()
	# load vertical axis properties, if present.
	vaxis = elem.getElementsByTagName(axis.VertAxis.axis_xml_name)
	if vaxis:
	    self.vaxis.from_xml(vaxis[0])
	# load horizontal axis properties, if present.
	haxis = elem.getElementsByTagName(axis.HorizAxis.axis_xml_name)
	if haxis:
	    self.haxis.from_xml(haxis[0])
	# load all plot objects.
	for plobj in elem.getElementsByTagName('plotobject'):
	    plotobject = create_plot_object_from_xml(plobj, self)
	    self.add_plot_object(plotobject)

    def real_button_press(self, event):
        if not self.select_area_mode:
	    return self.object_focus_button_press(event)
        if event.button != 1: return 1
        self.drag_origin = event.x, event.y
        self.selecting_area = 1
        return 1

    def real_motion_notify(self, event):
        if not self.selecting_area:
	    if self.select_area_mode:
		return 1
	    else:
		return self.object_focus_motion_notify(event.x, event.y)
	self.overlay.hide()
	x1, y1 = self.drag_origin
	x2, y2 = event.x, event.y
	if x1 > x2:
	    x1, x2 = x2, x1
	if y1 > y2:
	    y1, y2 = y2, y1
	self.overlay.set_bbox(int(x1), int(y1), int(x2 - x1), int(y2 - y1))
	self.overlay.show()
        return 1

    def real_button_release(self, event):
        if not self.selecting_area: return 0
        self.selecting_area   = 0
        self.select_area_mode = 0
        self.canvas.set_cursor(None)
	self.overlay.hide()
        self.view.control.get_ui_component().set_prop("/commands/ZoomArea",
						      "state", "0")
	if abs(event.x - self.drag_origin[0]) < 4: return 1
	if abs(event.y - self.drag_origin[1]) < 4: return 1
        pl = self.get_plotter()
	x0, y0 = pl.window2space(*self.drag_origin)
	x1, y1 = pl.window2space(event.x, event.y)
	if x0 > x1:
	    x0, x1 = x1, x0
	if y0 > y1:
	    y0, y1 = y1, y0
        self.view.set_zoom_area(self.bbox2range_x(x0),
                                self.bbox2range_y(y0),
                                self.bbox2range_x(x1),
                                self.bbox2range_y(y1))
	return 1

    def begin_zoom_select(self):
        self.canvas.set_cursor(gtk.gdk.Cursor(gtk.gdk.CROSSHAIR))
        self.select_area_mode = 1

    def cancel_zoom_select(self):
	self.overlay.hide()
        self.selecting_area   = 0
        self.select_area_mode = 0
        self.canvas.set_cursor(None)

    def get_y_range(self):
	ymin, ymax = None, None
	for obj in self.get_plot_objects():
	    mM = obj.get_y_range()
	    if mM is None: continue
	    m, M = mM
	    if ymin is None:
		ymin = m
		ymax = M
	    else:
		if m < ymin: ymin = m
		if M > ymax: ymax = M
	if ymin is not None:
	    return ymin, ymax

    def suggest_lineattr(self):
	minimum_distance = 1
	for lineattr in _predefined_lineattrs:
	    minimum = None
	    for existing in [obj.lineattr for obj in self.get_plot_objects()]:
		distance = LineAttr.distance(lineattr, existing)
		if minimum is None: minimum = distance
		else:               minimum = min(minimum, distance)
	    if minimum > minimum_distance:
		return lineattr
	#print "Warning: no predefined lineattr is different enough"
	return _predefined_lineattrs[0]

    def preselect_object(self, obj):
	if obj is self.preselected_object:
	    return
	if self.preselected_object is not None:
	    self.preselected_object.set_prelight(False)
	self.preselected_object = obj
	if self.preselected_object is not None:
	    self.preselected_object.set_prelight(True)

    def object_focus_motion_notify(self, x, y):
	closest = None
	closest_distance = None
	max_distance = 10
	for obj in self.get_plot_objects():
	    d = obj.distance_to_point(x, y)
	    if d > max_distance: continue # object is too far away; ignore.
	    if closest is None:
		closest_distance = d
		closest = obj
	    else:
		if d < closest_distance:
		    closest_distance = d
		    closest = obj
	self.preselect_object(closest)

    def display_preselected_properties(self, widget, obj):
	win = self.plotobjectlist.show_properties_dialog(obj)
	self.set_transient_for(win.dialog)
    def __menu_delete_plot_object(self, widget, obj):
	self.delete_plot_object(obj)

    def object_focus_button_press(self, event):
	# Note: the following test works because the implementation of
	# get_child_at_point ignores the plot objects.
	if self.get_child_at_point(int(event.x), int(event.y)) is not None:
	    return 0
	if self.preselected_object is None:
	    if event.button == 1:
		self.select_object(None)
	    return 0

	if event.button == 1:
	    self.select_object(self.preselected_object)

	elif event.button == 3: 

	    menu = gtk.Menu()

	    # Properties
	    item = gtk.ImageMenuItem(stock_id=gtk.STOCK_PROPERTIES)
	    item.show()
	    item.connect("activate", self.display_preselected_properties,
			 self.preselected_object)
	    menu.append(item)
	    # Delete
	    item = gtk.ImageMenuItem(stock_id=gtk.STOCK_DELETE)
	    item.show()
	    item.connect("activate", self.__menu_delete_plot_object,
			 self.preselected_object)
	    menu.append(item)

	    menu.popup(parent_menu_shell=None, parent_menu_item=None,
		       func=None, button=event.button,
		       activate_time=event.time)
	else:
	    return 0
	return 1

    def real_leave(self):
	self.preselect_object(None)

    def real_enter(self):
	x, y, mod = self.view.canvas.window.get_pointer()
	self.object_focus_motion_notify(x, y)

    def select_object(self, obj):
	if obj is self.selected_object:
	    return
	if self.selected_object is not None:
	    self.selected_object.set_selected(0)
	self.selected_object = obj
	if self.selected_object is not None:
	    self.selected_object.set_selected(1)
	    self.plotobjectlist.select_object(self.selected_object)
	self.emit("selection-changed", self.selected_object)

    def clear_selected(self):
	if self.selected_object is not None:
	    self.delete_plot_object(self.selected_object)

    def selected_properties(self):
	assert self.selected_object is not None
	win = self.plotobjectlist.show_properties_dialog(self.selected_object)
	self.set_transient_for(win.dialog)

    ## --- copy-paste ---
    __clipboard = None
    def edit_cut(self):
	assert self.selected_object is not None
	self.__clipboard = self.selected_object.to_xml_string()
	self.delete_plot_object(self.selected_object)
	rv = self.canvas.selection_owner_set(_SELECTION_CLIPBOARD,
					     time=0L)

    def edit_copy(self):
	assert self.selected_object is not None
	self.__clipboard = self.selected_object.to_xml_string()
	rv = self.canvas.selection_owner_set(_SELECTION_CLIPBOARD,
					     time=0L)
	assert rv, "Failed to claim ownership of the clipboard!"

    def __selection_received_cb(self, widget, selection_data, data):
	#print "selection-received:", selection_data, data
	#print "targets:", selection_data.get_targets()
	xml = selection_data.get_text()
	if xml is None:
	    print "Received empty selection; ignoring."
	    return
	doc = dom.parseString(xml)
	doc.getElementsByTagName("plotobject")
	for plobj in doc.getElementsByTagName('plotobject'):
	    plotobject = create_plot_object_from_xml(plobj, self)
	    self.add_plot_object(plotobject)

    def __selection_get_cb(self, widget, selection_data, info, time):
	assert self.__clipboard is not None
	print "*** selection-get:", self.__clipboard
	selection_data.set(_SELECTION_TYPE_SRING, 8, self.__clipboard)

    def edit_paste(self):
	self.canvas.selection_convert(_SELECTION_CLIPBOARD,
				      _SELECTION_XYGRAPH_TARGET,
				      time=0L)
	# __selection_received_cb will be called next

gobject.type_register(PlotArea)

try:
    import psyco
    psyco.bind(PlotArea.object_focus_motion_notify)
except ImportError:
    pass
