#! /usr/bin/env python

try:
    import gnomeprint
    import gnomeprint.ui
    from nxplot import *
    available = 1
except ImportError:
    available = 0

def get_page_area(job):
    return gnomeprint.job_get_page_size_from_config(job.get_config())

def do_print(control, dialog, job):
    pc = gnomeprint.Context(dialog.get_config())
    job.render(pc)
    pc.showpage()
    pc.close()


def render_to_job(control, job):
    pc = job.get_context()
    pl = NxplotGP(pc)
    area = get_page_area(job)
    pl.open()
    control.xygraph.render_to_page(pl, area)
    job.close()

def show_preview(control, dialog):
    job = gnomeprint.Job(dialog.get_config())
    render_to_job(control, job)
    w = gnomeprint.ui.JobPreview(job, "Print Preview")
    w.set_property('allow-grow', 1)
    w.set_property('allow-shrink', 1)
    w.set_transient_for(dialog)
    w.show_all()

def print_dialog_response(dialog, resp, data):
    control, job = data
    if resp == gnomeprint.ui.DIALOG_RESPONSE_PREVIEW:
	show_preview(control, dialog)
    elif resp == gnomeprint.ui.DIALOG_RESPONSE_CANCEL:
	dialog.destroy()
    elif resp == gnomeprint.ui.DIALOG_RESPONSE_PRINT:
	render_to_job(control, job)
	do_print(control, dialog, job)
	dialog.destroy()
    

def show_print_dialog(control):
    if not available: return
    job = gnomeprint.Job(gnomeprint.config_default())
    dialog = gnomeprint.ui.Dialog(job, "Print...", 0)
    if control is not None:
	control.set_transient_for(dialog)
    dialog.connect('response', print_dialog_response,
		   (control, job))
    dialog.show()



if __name__ == '__main__':
    show_print_dialog(None)


__all__ = (
    'show_print_dialog',
    'available',
    )

