from misc import *
from kernel import *
from plotobject_registry import *
from dialogs.registry import *
import warnings
import dialogs.util
import markers
import xml.dom.minidom as dom
import ORBit, CORBA
import xyconsts
def N_(message): return message


class Scatter(PlotObject):

    def __init__(self, values, plotarea):
        PlotObject.__init__(self)
        self.expression = None
        self.variable = None
	self.tree = None
	self.bbox = None
	self.samples = None
	self.marker_size = 0
	if type(values) is DictionaryType:
	    self.set_values(values)
	elif isinstance(values, dom.Element):
	    self.from_xml(values, plotarea)
	else:
	    raise TypeError("expected dictionary or DOM Element")
	self.__update_mathml()

        self.set_state(self.STATE_NORMAL)

    def get_values(self):
	values = PlotObject.get_values(self)
	values['contents'] = { 'expression': self.expression,
			       'marker':     self.marker.name,
			       'size': 	     self.marker_size,
			       }
	values['type'] = 'Scatter'
	return values

    def set_values(self, values):
	PlotObject.set_values(self, values)
	values = values['contents']
	self.expression = values['expression']
    	self.set_expression(self.expression)
	self.marker_size = values['size']

	name = values['marker']
	for marker in markers.all_markers:
	    print "marker name:", marker.name, "; name:", name
	    if marker.name == name:
		self.marker = marker
		print "New marker: ", self.marker
		break
	else:
	    warnings.warn("Marker name '%s' is not valid!" % (values['marker'],))
	self.__update_mathml()
	
    def __del__(self):
	if self.tree is not None:
	    kernel.elementUnref(self.tree)
	self.tree = None

    def update_samples(self):
	if self.canvas is None: return
	try:
	    self.sample()
	except CORBA.UserException, ex:
	    self.samples = None
	    err = str(ex.__class__) + ": " + ex.detail
	    dialog = gtk.MessageDialog(
		flags          = gtk.DIALOG_DESTROY_WITH_PARENT,
		type           = gtk.MESSAGE_ERROR,
		buttons        = gtk.BUTTONS_OK,
		message_format = err)
	    dialog.set_property("has-separator", gtk.FALSE)
	    self.view.set_transient_for(dialog)
	    dialog.run()
	    dialog.destroy()
	    return
	except Exception, ex:
	    self.samples = None
	    dialog = gtk.MessageDialog(
		flags          = gtk.DIALOG_DESTROY_WITH_PARENT,
		type           = gtk.MESSAGE_ERROR,
		buttons        = gtk.BUTTONS_OK,
		message_format = "Unknown error while sampling function:\n" +
		                 str(ex.__class__))
	    self.view.set_transient_for(dialog)
	    dialog.run()
	    dialog.destroy()
	    raise
	self.queue_draw()

	
    def set_range(self, x1, y1, x2, y2):
	PlotObject.set_range(self, x1, y1, x2, y2)
	self.update_samples()

    def set_expression(self, expression):
	self.expression = expression
	self.update_samples()
	    

    def sample(self):
	tree = kernel.parseString(self.expression)
	el = kernel.elementEval(tree)
	kernel.elementUnref(tree)
	res = kernel.elementAsAny(el).value()
	kernel.elementUnref(el)
	self.samples = map(lambda any: any.value(), res)
	self.samples = map(lambda pair: (pair[0].value(), pair[1].value()),
			   self.samples)


    def real_draw(self, pl):
	if not self.samples: return
	for lineattr in (self.outer_lineattr, self.inner_lineattr):
            if lineattr is None:
                continue
	    pl.endpath()
	    lineattr.apply_to(pl, xyconsts.WIDTH_SCALE)
	    marker = self.marker(self.calc_marker_radius(self.marker_size))
	    for (x, y) in self.samples:
		x1 = self.range2bbox_x(x)
		y1 = self.range2bbox_y(y)
		marker.draw(pl, x1, y1)
	    pl.endpath()

    def calc_marker_radius(self, size):
	return size*xyconsts.WIDTH_SCALE*5

    def draw_legend_sample(self, pl, really_draw):
	r = self.calc_marker_radius(self.marker_size)
	if not really_draw:
	    return 2*r, 2*r
	else:
	    pl.endpath()
	    self.lineattr.apply_to(pl, xyconsts.WIDTH_SCALE)
	    marker = self.marker(r)
	    marker.draw(pl, r, r)
	    pl.endpath()

    # override legend label drawing function to enable mathml
    def draw_legend_label(self, pl, attributes, really_draw, data=None):
	return self._draw_legend_label_mathml(self.mathml, pl, attributes,
					      really_draw, data)

    def __update_mathml(self):
	try:
	    el = kernel.parseString(self.expression)
	    expression_mathml = kernel.elementToMathML(el)
	    kernel.elementUnref(el)
	except ex:
	    expression_mathml = '<merror>error converting expression to MathML: %s</merror>' % str(ex)
	expression_mathml = expression_mathml.decode("utf-8")
	self.mathml = self._label_to_mathml(self.caption,
					    {'%e': expression_mathml})

    def to_xml(self, doc):
	assert isinstance(doc, dom.Document)
	elem = PlotObject.to_xml(self, doc)
	elem.setAttribute("type", "Scatter")
	expr = doc.createElement("expression")
	expr.appendChild(doc.createTextNode(self.expression))
	elem.appendChild(expr)
	marker = doc.createElement("marker")
	marker.setAttribute("size", str(self.marker_size))
	marker.setAttribute("type", self.marker.name)
	elem.appendChild(marker)
	return elem

    def from_xml(self, elem, plotarea):
	PlotObject.from_xml(self, elem, plotarea)
	assert elem.getAttribute("type") == 'Scatter'
	expr = elem.getElementsByTagName('expression')[0]
	self.expression = ""
	for node in expr.childNodes:
	    if node.nodeType == node.TEXT_NODE:
		self.expression += node.data
	self.expression = self.expression.strip().encode()
	marker = elem.getElementsByTagName('marker')[0]
	self.marker_size = float(marker.getAttribute("size"))
	marker_name = marker.getAttribute("type")
	for marker in markers.all_markers:
	    if marker.name == marker_name:
		self.marker = marker
		break
	else:
	    # Invalid marker name in XML file -> use the default one
	    self.marker = markers.MarkerX
	self.__update_mathml()
	    

    def get_control_points(self):
	if not self.samples: return ()
	pl = self.get_plotter()
	if not isinstance(pl, NxplotArtft): return ()
	rv = []
	min_distance = 32
	last_wx = None
	last_wy = None
	for x, y in self.samples:
	    try:
		wx, wy = pl.space2window(self.range2bbox_x(x),
					 self.range2bbox_y(y))
	    except AttributeError: return ()
	    if last_wx is None:
		rv.append((wx, wy))
		last_wx, last_wy = wx, wy
	    else:
		d = math.hypot(wx - last_wx, wy - last_wy)
		if d > min_distance:
		    rv.append((wx, wy))
		    last_wx, last_wy = wx, wy
	return rv


class ScatterContents(PlotObjectContents):
    def __init__(self):
	PlotObjectContents.__init__(self, "func contents scatter")
	self.name = "Scatter"
	self.expression_wid = dialogs.util.ExpressionEntry(kernel)
        self.expression_wid.show()
	self.xml.get_widget("expression container").add(self.expression_wid)
	self.xml.get_widget("expression label").set_mnemonic_widget(self.expression_wid)
	self.marker = markers.all_markers[0].name
	self._update_markers_menu()
	self.size_adj = self.xml.get_widget("scatter size").get_adjustment()
	
    def get_values(self):
	rv = {'expression': self.expression_wid.get_text(),
	      'marker':     self._get_selected_marker_from_menu().name,
	      'size':       self.size_adj.get_value(),
	      }
	return rv

    def set_values(self, values):
	self.expression_wid.set_text(values['expression'])
	self.marker = values['marker']
	self._update_markers_menu()
	self.size_adj.set_value(values['size'])

    def _update_markers_menu(self):
	menu, history = self._create_markers_menu(self.marker)
	opmenu = self.xml.get_widget("marker opmenu")
	opmenu.set_menu(menu)
	opmenu.set_history(history)

    def _get_selected_marker_from_menu(self):
	opmenu = self.xml.get_widget("marker opmenu")
	menu = opmenu.get_menu()
	item = menu.get_children()[opmenu.get_history()]
	return item.get_data("marker")

    def _create_markers_menu(self, selected_item):
	menu = gtk.Menu()
	index = -1
	history = 0
	for marker in markers.all_markers:
	    index += 1
	    item = gtk.MenuItem(marker.name)
	    menu.append(item)
	    item.show()
	    item.set_data("marker", marker)
	    if marker.name == selected_item:
		history = index
	return menu, history



plotobjectcontents_list.append(ScatterContents)
type_name_to_class_mapping[N_('Scatter')] = Scatter
__all__ = ['Scatter']
print "'Scatter' plot object has been registered"


