#! /usr/bin/env python
import gtk
from nxplot import *
from misc import *
from nxpruler import *
from copy import *
from dialogs.objectlist import *
from dialogs.util import *
from plotobject_registry import *
from plotarea import *
import math, types
import xml.dom.minidom as dom
import legend
import axis
import xyconsts
import gnumexp_conf
from gettext import gettext as _
import xygraphC




 ###############
### # # # # # ####
## # # # # # # ####
### # # # # # ####
 ###############

class FileFormatVersion:
    def __init__(self, string):
	self.version = [int(x) for x in string.split('.')]
    def __str__(self):
	return '.'.join(map(str, self.version))
    def __cmp__(self, other):
	return cmp(self.version, other.version)

this_file_format_version = FileFormatVersion('0.1')


class XYGraphView(object):
    def __init__(self):
	self.control = None
	self.canvas = canvas = NxpCanvas();
	self.modified = 0
	self.__modified_ignore = 0
	canvas.show()
	pl = self.pl = NxplotArtft(canvas)
	pl.space(0, 0, xyconsts.PLOT_AREA_WIDTH, xyconsts.PLOT_AREA_HEIGHT)
	canvas.set_plotter(pl)
	self.plot_area =  PlotArea(self)
	canvas.set_viewport(0, 0, xyconsts.PLOT_AREA_WIDTH, xyconsts.PLOT_AREA_HEIGHT)
	canvas.set_root(self.plot_area)
	self._create_ui()
	self.vruler.connect_after("size-allocate", self._vruler_configured)
	self.hruler.connect_after("size-allocate", self._hruler_configured)
	self.width, self.height = (0, 0)
	self.plot_area.connect("selection-changed", self.__selection_changed_cb)


    def _hruler_configured(self, *args):
	self.__modified_ignore = 1
	self._hrange_changed(self.hadj)
	self.__modified_ignore = 0

    def _vruler_configured(self, *args):
	self.__modified_ignore = 1
	self._vrange_changed(self.vadj)
	self.__modified_ignore = 0
	

    def _range_changed(self):
	self.plot_area.set_range(self.hadj.lower, self.vadj.lower,
				 self.hadj.upper, self.vadj.upper)
	self.set_modified(1)
	
    def _create_ui(self):

	self.hadj = hadj = NxpRulerAdjustment(-10, 10)
	self.vadj = vadj = NxpRulerAdjustment(-10, 10)
	# A "range_changed" signal is emmited whenever the ruler is
	#   scrolled or stretched.
	self.hadj.connect_after('range_changed', self._hrange_changed)
	self.vadj.connect_after("range_changed", self._vrange_changed)

	# table
	self.table = table = gtk.Table(3, 2, 0)
	table.show()

	# hruler
	self.hruler = hruler = NxpHRuler(self.hadj); hruler.show()
	table.attach(hruler, 1, 2, 0, 1, gtk.EXPAND|gtk.FILL, 0, 0, 0)
	self.hruler.connect('button-press-event', self.__hruler_button_press)

	# vruler
	self.vruler = vruler = NxpVRuler(vadj); vruler.show()
	table.attach(vruler, 0, 1, 1, 2, 0, gtk.EXPAND|gtk.FILL, 0, 0);
	self.vruler.connect('button-press-event', self.__vruler_button_press)

	# add canvas
	table.attach(self.canvas, 1, 2, 1, 2)
	self.canvas.connect("motion_notify_event", self._canvas_motion_notify)
	self.last_allocation = None
	self.canvas.connect_after("size-allocate", self._canvas_size_allocate)
	self.hruler.connect_after("size-allocate", self._hruler_size_allocate)
	self.vruler.connect_after("size-allocate", self._vruler_size_allocate)

    def _canvas_size_allocate(self, canvas, allocation):
	if (self.last_allocation is not None and
	    self.last_allocation[0] == allocation.width and
	    self.last_allocation[1] == allocation.height): return
	self.last_allocation = allocation.width, allocation.height
	k = float(allocation.width) / allocation.height
	h = xyconsts.PLOT_AREA_HEIGHT
	w = h*k
	self.pl.space(0, 0, w, h)
	self.canvas.set_viewport(0, 0, w, h)
	self.width, self.height = (w, h)

    def _canvas_motion_notify(self, canvas, event):
	self.hruler.event(event)
	self.vruler.event(event)
	return 0

    def __hruler_size_allocate_idle(self):
	self.plot_area.set_hticks(self.hruler.rtick_start, self.hruler.rtick_div,
				  self.hruler.rtick_subdiv, self.hruler.rtick_limit)
    def __vruler_size_allocate_idle(self):
	self.plot_area.set_vticks(self.vruler.rtick_start, self.vruler.rtick_div,
				  self.vruler.rtick_subdiv, self.vruler.rtick_limit)
    def _hruler_size_allocate(self, hruler, *args):
	gobject.idle_add(self.__hruler_size_allocate_idle)
    def _vruler_size_allocate(self, vruler, *args):
	gobject.idle_add(self.__vruler_size_allocate_idle)


    def _hrange_changed(self, adj):
	#print "Horizontal range changed: new range = [%f, %f]" % (adj.lower, adj.upper)
	self._range_changed()
	self.plot_area.set_hticks(self.hruler.rtick_start, self.hruler.rtick_div,
				  self.hruler.rtick_subdiv, self.hruler.rtick_limit)

    def _vrange_changed(self, adj):
	#print "Horizontal range changed: new range = [%f, %f]" % (adj.lower, adj.upper)
	self._range_changed()
	self.plot_area.set_vticks(self.vruler.rtick_start, self.vruler.rtick_div,
				  self.vruler.rtick_subdiv, self.vruler.rtick_limit)
	
    def get_toplevel_widget(self):
	return self.table

    
    def show_plot_objects_list_window(self):
	return self.plot_area.show_plot_objects_list_window()

    def show_add_object_dialog(self):
	win = self.plot_area.plotobjectlist.model.create_new_object_dialog()
	self.plot_area.provide_new_plot_object_defaults(win)
	return win

    def zoom_in(self):
	x1, y1, x2, y2 = self.hadj.lower, self.vadj.lower,\
			 self.hadj.upper, self.vadj.upper
	dx, dy = x2 - x1, y2 - y1
	marginx, marginy = dx/4, dy/4
	x1 += marginx
	y1 += marginy
	x2 -= marginx
	y2 -= marginy
	self.hadj.set_range(x1, x2)
	self.vadj.set_range(y1, y2)

    def zoom_out(self):
	x1, y1, x2, y2 = self.hadj.lower, self.vadj.lower,\
			 self.hadj.upper, self.vadj.upper
	dx, dy = x2 - x1, y2 - y1
	marginx, marginy = dx/2, dy/2
	x1 -= marginx
	y1 -= marginy
	x2 += marginx
	y2 += marginy
	self.hadj.set_range(x1, x2)
	self.vadj.set_range(y1, y2)

    def set_zoom_area(self, x1, y1, x2, y2):
        self.hadj.set_range(x1, x2)
	self.vadj.set_range(y1, y2)


    def zoom_area(self, state):
	if state:
	    self.plot_area.begin_zoom_select()
	else:
	    self.plot_area.cancel_zoom_select()

    def set_grid_active(self, active):
	self.plot_area.vticks_enabled = active
	self.plot_area.hticks_enabled = active
	self.plot_area.queue_draw()

    def set_axis_active(self, active):
	self.plot_area.set_vaxis_enabled(active)
	self.plot_area.set_haxis_enabled(active)
	self.plot_area.queue_draw()

    def set_legend_active(self, active):
	self.plot_area.set_legend_enabled(active)
	self.plot_area.queue_draw()
    def get_legend_active(self):
	return self.plot_area.get_legend_enabled()

    def save_to(self, savefile):
	"""save xml to a file-like object @savefile"""

	xygraphC.reset_lc_numeric() # see comment in XYGraphView::load_from
	doc  = dom.Document()
	xy = doc.createElement("xygraph")
	xy.setAttribute("version", str(this_file_format_version))
	xy.setAttribute("creator", "gnumexp " + gnumexp_conf.VERSION)
	plarea = self.plot_area.to_xml(doc)
	xy.appendChild(plarea)
	doc.appendChild(xy)
	doc.writexml(savefile, addindent="    ", newl="\n")
	self.modified = 0

    def load_from(self, loadfile):
	"""read xml from a file-like object @loadfile"""

	# somehow, LC_NUMERIC category is set to the current locale,
	# at the libc level, which causes all sorts of breakage if the
	# locale uses comma instead of point as decimal
	# separator.  Portuguese and german, for example, use this
	# convention.  Here, we call a C function that calls
	# setlocale(LC_NUMERIC, "C").
	xygraphC.reset_lc_numeric()

	doc = dom.parse(loadfile)
	root = doc.documentElement
	if root.tagName != "xygraph":
	    raise RuntimeError("This is not a gnumexp plot file!")

	# Check if we can read this format version
	ver = root.getAttribute('version')
	if ver:
	    if FileFormatVersion(ver) > this_file_format_version:
		msg = _("This file was created with a more recent version of gnumexp.\n"\
			"The file will be read anyway, but there will be at least"\
			" some information loss.")
		dialog = gtk.MessageDialog(
		    flags          = gtk.DIALOG_DESTROY_WITH_PARENT,
		    type           = gtk.MESSAGE_WARNING,
		    buttons        = gtk.BUTTONS_OK,
		    message_format = msg)
		self.set_transient_for(dialog)
		dialog.show()
		dialog.connect("response", lambda d, r: d.destroy())
	
	plotareas = root.getElementsByTagName('plotarea')
	if len(plotareas) == 0:
	    raise RuntimeError("File is broken: missing <plotarea> element.")
	if len(plotareas) > 1:
	    raise RuntimeError("File is broken: too many <plotarea> elements.")
	pa = plotareas[0]
	self.from_xml_plotarea(pa)

    def from_xml_plotarea(self, pa, replace=1):
	self.plot_area.from_xml(pa, replace)
	self.__modified_ignore = 1
	# get x range, if any
	x1 = pa.getAttribute("x1")
	x2 = pa.getAttribute("x2")
	if x1 and x2:
	    self.hadj.set_range(float(x1), float(x2))
	# get y range, if any
	y1 = pa.getAttribute("y1")
	y2 = pa.getAttribute("y2")
	if y1 and y2:
	    self.vadj.set_range(float(y1), float(y2))
	self.modified = 0
	self.__modified_ignore = 0


    def refresh(self):
	x1, y1, x2, y2 = self.hadj.lower, self.vadj.lower,\
			 self.hadj.upper, self.vadj.upper
	self.hadj.set_range(x1, x2)
	self.vadj.set_range(y1, y2)

    def set_transient_for(self, win):
	if self.control is not None:
	    self.control.set_transient_for(win)

    def set_modified(self, modified):
	if self.__modified_ignore: return
	if (self.modified and modified or
	    not self.modified and not modified):
	    return
	self.modified = modified
	self.control.modified_status_changed()
	
    def get_plot_objects(self):
	return copy(self.plot_area.plotobjectlist.plot_objects)

    def render_to_page(self, pl, area):
 	page_width, page_height = area
	page_ewidth = page_width*.9	# effective page width
	page_eheight = page_height*.9	# effective page height
	
	canvas_width  = self.width
	canvas_height = self.height
	
	if page_width <= page_height:
	    scale = page_ewidth / canvas_width
	else:
	    scale = page_eheight / canvas_height

	# destination rectangle
	dest_width  = scale*canvas_width
	dest_height = scale*canvas_height

	# Especially with landscape mode, these dimmensions can still
	# go outside the paper, so we may need to readjust the scale
	# factor.
	if dest_width > page_ewidth:
	    # Need to limit by width
	    scale = page_ewidth / canvas_width
	    dest_width  = scale*canvas_width
	    dest_height = scale*canvas_height
	elif dest_height > page_eheight:
	    # Need to limit by height
	    scale = page_eheight / canvas_height
	    dest_width  = scale*canvas_width
	    dest_height = scale*canvas_height
	
	# Offsets to center on page
	offset_x = page_width/2 - dest_width/2
	offset_y = page_height/2 - dest_height/2

	# draw rectangle
 	x1, y1 = offset_x, offset_y
 	x2, y2 = x1 + dest_width, y1
 	x3, y3 = x2, y2 + dest_height
 	x4, y4 = x1, y3
	pl.linewidth(1)
 	pl.pencolor(0, 0, 0)
 	pl.line(x1, y1, x2, y2)
 	pl.line(x2, y2, x3, y3)
	pl.line(x3, y3, x4, y4)
 	pl.line(x4, y4, x1, y1)
 	pl.endpath()

	# setup transformation
	pl.translate(offset_x, offset_y)
	pl.scale(scale, scale)

	# render the canvas
	pl.setcliprect(NxplotRect(0, 0, canvas_width, canvas_height))
	self.canvas.draw_to(pl)

	return

    def auto_vertical_fit(self):
	mM = self.plot_area.get_y_range()
	#print "self.plot_area.get_y_range():", mM
	if mM is None: return
	ymin, ymax = mM
	dy = (ymax - ymin)/10
	if dy == 0: return
	self.vadj.set_range(ymin - dy, ymax + dy)

    def __selection_changed_cb(self, plotarea, selection):
	if self.control is not None:
	    self.control.selection_changed(selection)
	

    __haxis_properties_dialog = None
    __vaxis_properties_dialog = None
    def __axis_properties_dialog_destroyed(self, dialog):
	if dialog is self.__haxis_properties_dialog:
	    self.__haxis_properties_dialog = None
	elif dialog is self.__vaxis_properties_dialog:
	    self.__vaxis_properties_dialog = None
	else:
	    assert 0, "unknown dialog destroyed"
    def __create_axis_properties_dialog_common(self, axis_Name, axis_pos_coord, pos, lower, upper):
	dialog = gtk.Dialog(title   = _("%s axis properties") % (axis_Name,),
			    flags   = gtk.DIALOG_DESTROY_WITH_PARENT,
			    buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
				       gtk.STOCK_OK,     gtk.RESPONSE_ACCEPT)
			    )
        dialog.set_default_response(gtk.RESPONSE_ACCEPT)
	dialog.set_property("has-separator", 0)
	dialog.set_border_width(12)
	self.set_transient_for(dialog)
	dialog.vbox.set_spacing(18)
	dialog.connect("destroy", self.__axis_properties_dialog_destroyed)

	# add content
	## -> position
	hbox = gtk.HBox()
	label = gtk.Label("%s = " % axis_pos_coord)
	entry = gtk.Entry()
	entry.set_text(pos)
	hbox.add(label)
	hbox.add(entry)
	hbox.show_all()
	hbox.set_spacing(6)
	dialog.vbox.add(create_indenting_container(_("Position"), hbox))
	dialog.position_entry = entry
        entry.set_activates_default(True)

	## -> limits
	vbox = gtk.VBox();
	vbox.show(); vbox.set_spacing(6)

	hbox = gtk.HBox(); hbox.set_spacing(6)
	label = gtk.Label(_("Lower"))
	entry = gtk.Entry()
	dialog.lower_limit = entry
	entry.set_text(lower);
	hbox.add(label)
	hbox.add(entry)
	hbox.show_all()
	vbox.add(hbox)
        entry.set_activates_default(True)

	hbox = gtk.HBox(); hbox.set_spacing(6)
	label = gtk.Label(_("Upper"))
	entry = gtk.Entry()
	dialog.upper_limit = entry
	entry.set_text(upper);
	hbox.add(label)
	hbox.add(entry)
	hbox.show_all()
	vbox.add(hbox)
        entry.set_activates_default(True)
	
	dialog.vbox.add(create_indenting_container(_("Limits"), vbox))

	dialog.show()
	return dialog

    def __axis_properties_validate(self, dialog, response):
	close_dialog = 1
	rv = None
	if response == gtk.RESPONSE_ACCEPT:
	    try:
		pos = float(dialog.position_entry.get_text())
		lower = float(dialog.lower_limit.get_text())
		upper = float(dialog.upper_limit.get_text())
		if lower >= upper:
		    raise ValueError(_("Lower limit is not less than upper limit"))
		rv = pos, lower, upper
	    except Exception, ex:
		errdlg = gtk.MessageDialog(
		    flags          = gtk.DIALOG_DESTROY_WITH_PARENT,
		    parent         = dialog,
		    type           = gtk.MESSAGE_ERROR,
		    buttons        = gtk.BUTTONS_OK,
		    message_format = str(ex))
		errdlg.run()
		errdlg.destroy()
		close_dialog = 0
		rv = None
	# maybe close dialog and return
	if close_dialog:
	    dialog.destroy()
	return rv

    def __haxis_properties_dialog_response(self, dialog, response):
	values = self.__axis_properties_validate(dialog, response)
	if values is not None:
	    pos, lower, upper = values
	    self.plot_area.haxis.change_position(pos)
	    self.hadj.set_range(lower, upper)
    def __vaxis_properties_dialog_response(self, dialog, response):
	values = self.__axis_properties_validate(dialog, response)
	if values is not None:
	    pos, lower, upper = values
	    self.plot_area.vaxis.change_position(pos)
	    self.vadj.set_range(lower, upper)

    def create_haxis_properties_dialog(self):
	if self.__haxis_properties_dialog is not None:
	    self.__haxis_properties_dialog.present()
	    return
	dialog = self.__create_axis_properties_dialog_common(
	    _("Horizontal"), "Y", str(self.plot_area.haxis.pos_fn),
	    str(self.hadj.lower), str(self.hadj.upper))
	dialog.connect("response", self.__haxis_properties_dialog_response)
	self.__haxis_properties_dialog = dialog
    def create_vaxis_properties_dialog(self):
	if self.__vaxis_properties_dialog is not None:
	    self.__vaxis_properties_dialog.present()
	    return
	dialog = self.__create_axis_properties_dialog_common(
	    _("Vertical"), "X", str(self.plot_area.vaxis.pos_fn),
	    str(self.vadj.lower), str(self.vadj.upper))
	dialog.connect("response", self.__vaxis_properties_dialog_response)
	self.__vaxis_properties_dialog = dialog

    def __hruler_button_press(self, hruler, event):
	if event.button != 2:
	    return 0
	self.create_haxis_properties_dialog()
    def __vruler_button_press(self, vruler, event):
	if event.button != 2:
	    return 0
	self.create_vaxis_properties_dialog()
